<?php

namespace api\controllers;

use api\controllers\BaseController;
use common\classes\Common;
use common\classes\Error;
use common\models\BaseInfo;
use common\models\BroadcastMode;
use common\models\DevClasses;
use common\models\DeviceList;
use common\models\Event;
use common\models\PlaySessions;
use common\models\PwrDisplaySet;
use common\models\SystemLog;
use common\models\TaskTemplate;
use common\models\UserDevices;
use common\models\Users;
use common\models\VDevice;
use common\models\VMediaFiles;
use common\models\VSessionInfo;
use yii;

class DashboardController extends BaseController
{
    public function actionDevStatStatistics()
    {
        //是否有分区(是否是简易模式)
        $ret = ['rf'=>[],'regdev'=>[],'tree_list'=>[]];
        $devquerry = VDevice::find()->select(['id', 'iotStatus', 'rf_invalid'])->where(['not', ['ipaddress'=>null]])
                      ->andWhere(['not like', 'id', ['49%', '48%', '40%'], false]);
        $claquerry = DevClasses::find()->select(['ClassID', 'Name']);
        $tree_list = DeviceList::getDevTreeArray();
        $classes = null;
        $data = Yii::$app->request->post('data');
        if(empty($data) || !isset($data['parent']) || $data['parent'] === 0) {
            $classes = $claquerry->Where(['Parent'=>null])->all();
            $classes[] = ['ClassID'=>-1, 'Name'=>null];     //parent为0或者不传，表示浏览所有顶层分区和不在任何分区的设备；
        }else{
            if (array_key_exists($data['parent'], $tree_list)) {
                $devlist = $tree_list[$data['parent']];
                $devquerry->andWhere(['in', 'id', $devlist]);
                $classes = $claquerry->where(['Parent' => $data['parent']])->all();
                $classes[] = $claquerry->where(['ClassID' => $data['parent']])->one();
            }else
                return $this->response(Error::STATUS_SUCCESS, $ret);
        }

        //各物联设备状态统计
        $rf = PwrDisplaySet::find()->select(['PwrID', 'Name', 'Type'])->where(['not in', 'PwrID', [0, 1]])
            ->andwhere(['not', ['Type'=>null]])->all();
        if (count($rf) > 0) {
            foreach ($rf as $r)
                $ret['rf'][] = ['name' => $r['Name'],'PwrID'=>$r['PwrID'], 'on' => 0, 'off' => 0, 'online' => 0, 'offline' => 0, 'no-power'=>0];
            $devices = $devquerry->all();
            if (count($devices) > 0) {
                foreach ($devices as $rfstat) {
                    foreach ($rf as $i => $r) {
                        $set = $r->PwrID;
                        $everyone = hexdec(substr($rfstat->iotStatus, $set * 2, 2));
                        if (($everyone & 0x8) != 0) {
                            $ret['rf'][$i]['online'] += 1;
                            if (($everyone & 0x7) != 0) {
                                $ret['rf'][$i]['on'] += 1;
                                if (($everyone & 0x80) == 0 && $r['Type'] != 2)
                                    $ret['rf'][$i]['no-power'] += 1;
                            }
                            else
                                $ret['rf'][$i]['off'] += 1;
                        } else if((($rfstat->rf_invalid>>$set) & 1) === 0)
                            $ret['rf'][$i]['offline'] += 1;
                    }
                }
            }
        }

        //各分区设备状态统计
        if (count($classes) > 0) {
            foreach ($classes as $class)
                $ret['regdev'][] = ['name' => $class['Name'], 'online' => 0, 'on' => 0, 'off' => 0, 'disable' => 0, 'offline' => 0];
            foreach ($classes as $idx => $class) {
                $status = null;
                $sql = "select power, count(*) as cnt from v_device";
                if (array_key_exists($class['ClassID'], $tree_list)) {
                    $device = $tree_list[$class['ClassID']];
                    $ids = implode(',', $device);
                    $sql .= " where id in ($ids)";

                } else if ($class['ClassID'] == -1) {
                    $sql .= " where parent is null";
                } else
                    continue;
                $sql .= " group by power";
                $items = Yii::$app->db->createCommand($sql)->queryAll();
                foreach ($items as $item) {
                    if(is_null($item['power']))
                        $ret['regdev'][$idx]['offline'] = $item['cnt'] + 0;
                    else
                        $ret['regdev'][$idx][$item['power']] = $item['cnt'] + 0;
                }
                $ret['regdev'][$idx]['online'] = $ret['regdev'][$idx]['on'] +
                    $ret['regdev'][$idx]['off'] +
                    $ret['regdev'][$idx]['disable'];
            }
        }

        //$data['parent']各个子分区设备列表
        if(!empty($data) && isset($data['parent'])) {
            $sort = 'name';
            if(isset($data['sort']))
                $sort = $data['sort'] == 'local' ? 'description': 'name';
            $querry2 = VDevice::find()->select(['id', 'name', 'local'=>'description', 'status'=>'power', 'ipaddress'])
                ->orderBy([$sort => SORT_ASC]);
            $pri_ids = [];
            $user = Yii::$app->user->identity;
            if ($user->Mgr_Privilege != 1 && !$this->isSuper){
                $ids = UserDevices::find()->select(['DevID'])->where(['UserID'=>$user->User_ID])
                ->andWhere('Pri & 2 = 2')->all();
                foreach ($ids as $d)
                    $pri_ids[] = $d['DevID'];
                if(empty($pri_ids))
                    $pri_ids[] = '-1';
            }
            foreach ($classes as $class) {
                $devstat = null;
                if($class['ClassID'] == $data['parent'])
                    $querry2->where(['parent' => $class['ClassID']]);
                else if (array_key_exists($class['ClassID'], $tree_list))
                    $querry2->where(['id' => $tree_list[$class['ClassID']]]);
                else if ($class['ClassID'] == -1)
                    $querry2->where(['parent' => null]);
                else
                    continue;
                if ($pri_ids)
                    $devstat = $querry2->andWhere(['in', 'id', $pri_ids])->asArray()->all();
                else
                    $devstat = $querry2->asArray()->all();
                foreach ($devstat as $id =>$dev){
                    if($dev['ipaddress'] == null)
                        $devstat[$id]['status'] = 'offline';
                }
                $ret['tree_list'][] = ['name'=>$class['Name'], 'device'=>$devstat];
            }
        }

        return $this->response(Error::STATUS_SUCCESS, $ret);
    }

    public function actionMediaStatistics () {
        $total = VMediaFiles::find()->select('FileSize')->where('Sysfolder is null and FilePath is not null');
        // total
        $totalCount = $total->count();
        $totalSize = 0;
        $totalFiles = $total->all();
        foreach ($totalFiles as $file) {
            $totalSize += $file->FileSize;
        }

        // audio
        $audio = VMediaFiles::find()->select('FileSize')->where('Sysfolder is null and FilePath is not null')
            ->andwhere('AudioCodec <> -1 and VedioCodec = -1');
        $audioCount = $audio->count();
        $audioSize = 0;
        $audioFiles = $audio->all();
        foreach ($audioFiles as $file) {
            $audioSize += $file->FileSize;
        }

        // video
        $video =VMediaFiles::find()->select('FileSize')->where('Sysfolder is null and FilePath is not null')
            ->andwhere(['or', ['VedioCodec' => [0,1,2], 'AudioCodec' =>[-1,1,2,3]], ['VedioCodec' => -1, 'AudioCodec' => -1]]);
        $videoCount = $video->count();
        $videoSize = 0;
        $videoFiles = $video->all();
        foreach ($videoFiles as $file) {
            $videoSize += $file->FileSize;
        }

        return $this->response(Error::STATUS_SUCCESS, [
            'total' => ['count' => $totalCount, 'size' => $totalSize],
            'audio' => ['count' => $audioCount, 'size' => $audioSize],
            'video' => ['count' => $videoCount, 'size' => $videoSize]
        ]);
    }

    public function actionSessionStatistics() {
        $querry = PlaySessions::find();
        //总任务
        $sessions_total = $querry->count();
        //控制
        $ctrol = $querry->where(['SessionType'=>[0, 2], 'StreamType'=>0]);
        $ctrl_total = $ctrol->count();
        //广播
        $bord_total = $sessions_total -  $ctrl_total;
        //定时
        $bord_auto = $querry->Where(['SessionType'=>[0, 8]]);
        $bord_auto_count = $bord_auto->count();
        //手动
        $bord_hanel = $querry->Where(['SessionType'=>[2, 9]]);
        $bord_handel_count = $bord_hanel->count();
        //实时
        $realtime = $querry->Where(['SessionType'=>[1, 10]]);
        $realtime_count = $realtime->count();
        //任务模式，和正在运行的模式
        $querry = BroadcastMode::find();
        $model_total = $querry->count();
        $running_count = $querry->where(['Active'=>1])->count();

        //新增任务
        $now = date("Y-m-d");
        $new_session_count = SystemLog::find()->where(['type' => 'session', 'action' => 'new', 'result' => 0])
            ->andWhere("'$now'=DATE_FORMAT(at, '%Y-%m-%d')")
            ->count();
        //当前任务模板
        $tempate_count = TaskTemplate::find()->count();

        return $this->response(Error::STATUS_SUCCESS, [
            'all_session'=>['all'=>$sessions_total + 0,
                            'ctrl_session'=>$ctrl_total + 0, 'bord_session'=>$bord_total + 0,
                            'new_session' => $new_session_count + 0,
                            'auto'=>$bord_auto_count + 0, 'handel'=>$bord_handel_count + 0, 'realtime'=>$realtime_count + 0],
            'model' => ['all'=> $model_total + 0, 'running'=>$running_count + 0],
            'template' => ['all'=>$tempate_count + 0]
        ]);
    }

    public function actionUserStatistics () {
        $sql = 'select count(*) as cnt from Users where User_ID > 0';
        // total
        $total = Yii::$app->db->createCommand($sql)->queryOne();
        $totalCount = $total['cnt'];
        // manager
        $manager = Yii::$app->db->createCommand($sql . ' and User_Name is not null')->queryOne();
        $managerCount = $manager['cnt'];
        // ic
        $ic = Yii::$app->db->createCommand($sql . ' and IcCard is not null')->queryOne();
        $icCount = $ic['cnt'];
        // phone
        $phone = Yii::$app->db->createCommand($sql . ' and Phone is not null')->queryOne();
        $phoneCount = $phone['cnt'];
        // temp
        $temp = Yii::$app->db->createCommand($sql . ' and User_Type = 3')->queryOne();
        $tempCount = $temp['cnt'];

        return $this->response(Error::STATUS_SUCCESS, [
            'total' => $totalCount,
            'manager' => $managerCount,
            'ic' => $icCount,
            'phone' => $phoneCount,
            'temp' => $tempCount
        ]);
    }

    public function actionLogStatistics () {
        $query = Yii::$app->db
            ->createCommand('SELECT Event, COUNT(*) AS cnt from DeviceLog GROUP BY `Event`');
        $logs = $query->queryAll();
        $devLogCount = 0;
        $userLogCount = 0;
        $authLogCount = 0;
        foreach ($logs as $log) {
            if ($log['Event'] == 2)
                $userLogCount += $log['cnt'];
            if ($log['Event'] == 11 || $log['Event'] == 12)
                $authLogCount += $log['cnt'];
            $devLogCount += $log['cnt'];
        }

        $sysLog = Yii::$app->db->createCommand('SELECT COUNT(*) AS cnt from SystemLog')->queryOne();
        $sysLogCount = $sysLog['cnt'];
        return $this->response(Error::STATUS_SUCCESS, [
            'total' => $devLogCount + $userLogCount + intval($sysLogCount) + $authLogCount,
            'device' => $devLogCount,
            'user' => $userLogCount,
            'system' => intval($sysLogCount),
            'auth' => $authLogCount
        ]);
    }

    public function actionSystemStatistics () {
        // ram
        $memory = json_decode(stripcslashes(shell_exec(  "systeminfo current_ram")),true);
        $memory['total'] = floor($memory['total']);
        $memory['used'] = floor($memory['used']);
        $memory['usedRate'] = floor($memory['used'] * 100 / $memory['total']);

        // disk
        $metaDisks = json_decode(stripcslashes(shell_exec(  "systeminfo disk_partitions")),true);
        $disks = [];
        foreach ($metaDisks as $disk) {
            if(!preg_match('/\/dev\//', $disk['file_system']))
                continue;
            $disks[] = ['dev' => $disk['file_system'], 'size' => $disk['size'], 'rate' => $disk['used%']];
        }

        // remote support
        $support = !is_null(Yii::$app->base->support);
        // op time
        $tmp = json_decode(stripcslashes(shell_exec(  "systeminfo general_info")),true);
        $upTime = $tmp['Uptime'];
        $upTime = str_replace('days', Yii::t('app', 'days'), $upTime);
        $upTime = str_replace('hours', Yii::t('app', 'hours'), $upTime);
        $upTime = str_replace('minutes', Yii::t('app', 'minutes'), $upTime);
        $upTime = str_replace('seconds', Yii::t('app', 'seconds'), $upTime);
        $upTime = str_replace('and ', '', $upTime);
        $upTime = trim($upTime);
        // cloud
        $cloudInfo = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_CLOUD_STATUS]);
        if(empty($cloudInfo) || empty($cloudInfo->Description)) {
            $cloudStatus = 'error';
        }
        else {
            $cloudStatus = $cloudInfo->Description;
        }
        // template
        $model = TaskTemplate::findOne(['idTaskTemplate' => Yii::$app->base->activeTemplateId]);
        if(empty($model)) {
            $template = Yii::t('app', 'unset');
        }
        else if(empty($model->TemplateName)) {
            if($model->idTaskTemplate == 0)
                $template = Yii::t('app', 'default');
            else
                $template = Yii::t('app', 'default') . " copy";
        }
        else
            $template = $model->TemplateName;
        // database backup
        $bi = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_MYSQL_BACKUP_INFO]);
        $dbBackup = '';
        if(!empty($bi) && !empty($bi->Description))
            $dbBackup = Yii::t('app', 'db_backup_success') . pathinfo($bi->Description, PATHINFO_FILENAME);
        return $this->response(Error::STATUS_SUCCESS, [
            'memory' => $memory,
            'disk' => $disks,
            'others' => [
                'support' => $support,
                'upTime' => $upTime,
                'cloud' => $cloudStatus,
                'template' => $template,
                'dbBackup' => $dbBackup
            ]
        ]);
    }

    public function actionAuthLogStatistics () {
        $query = Yii::$app->db->createCommand('select Event, count(*) as cnt from v_device_log where Event in (11, 12) group by Event');
        $datas = $query->queryAll();
        $success = 0;
        $failed = 0;
        foreach ($datas as $log) {
            if ($log['Event'] == 11)
                $success = $log['cnt'];
            if ($log['Event'] == 12)
                $failed = $log['cnt'];
        }
        return $this->response(Error::STATUS_SUCCESS, ['success' => $success, 'failed' => $failed]);
    }

    public function actionSyncStatistics()
    {
        $data = Yii::$app->request->post('data');
        if(empty($data)) {
            if(empty($data['page']))
                $data['page'] = 0;
            if(empty($data['limit']))
                $data['limit'] = 2;
            if(empty($data['key']))
                $data['key'] = ['ic-sync', 'timetable-sync'];
        }

        $ret = ['ic-sync'=>[], 'timetable-sync'=>[]];
        $offset = $data['page'] * $data['limit'];

        foreach ($data['key'] as $type){
            $sync =  SystemLog::find()->select(['at', 'success' => 'result', 'sync_num' => 'title1'])->where(['action' => $type])
                ->offset($offset)
                ->limit($data['limit'])
                ->asArray()
                ->all();
            if(!empty($sync)) {
                foreach ($sync as $idx=>$ic) {
                    if($ic['success'] == '0')
                        $icsync[$idx]['success'] = true;
                    else
                        $icsync[$idx]['success'] = false;
                }
                $ret[$type] = $sync;
            }
        }

        return $this->response(Error::STATUS_SUCCESS, $ret);
    }

    //状态实时总览
    public function actionStatusRefresh(){
        $data = Yii::$app->request->post('data');
        if(empty($data) || empty($data['parent']) || !is_numeric($data['parent']))
            $parent = 0;
        else
            $parent = $data['parent'];

        $tree_list = DeviceList::getDevTreeArray();
        $new_status = ['online'=>0, 'on'=>0, 'off'=>0, 'disable'=>0, 'offline'=>0, 'parent'=>$parent];
        if (!empty($tree_list[$parent])) {
            $devices = $tree_list[$parent];
            $ids = implode(',', $devices);
            $device = Yii::$app->db->createCommand("select power, count(*) as cnt from v_device where id in ($ids) group by power")->queryAll();
            foreach ($device as $dev) {
                if(is_null($dev['power']))
                    $new_status['offline'] = $dev['cnt'] + 0;
                else
                    $new_status[$dev['power']] = $dev['cnt'] + 0;
            }
            $new_status['online'] = $new_status['on'] + $new_status['off'] + $new_status['disable'];
            return $this->response(Error::STATUS_SUCCESS, $new_status);
        }

        return $this->response(Error::STATUS_SUCCESS, $new_status);
    }
}
