<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/5/2 15:34
 */
namespace api\controllers;
use common\classes\Common;
use common\classes\Error;
use common\models\VDevice;
use common\models\VDeviceLog;
use common\models\VDeviceStatistic;
use Yii;

/**
 * Site controller
 */
class LogController extends BaseController
{
    private $startDate, $endDate;
    private $dateRange = [];
    public function beforeAction($action)
    {
        $data = Yii::$app->request->post('data');
        if(!$data || !key_exists('startDate', $data) || strptime($data['startDate'], '%Y-%m-%d') == false) {
            $start = Yii::$app->request->get('startDate');
            if(!$start || strptime($start, '%Y-%m-%d') == false)
                $this->startDate = date("Y-m-d", (time()  - 86400 * 30));
            else
                $this->startDate = $start;

        }
        else
            $this->startDate = $data['startDate'];

        if(!$data || !key_exists('endDate', $data) || strptime($data['endDate'], '%Y-%m-%d') == false) {
            $end = Yii::$app->request->get('endDate');
            if (!$end || strptime($end, '%Y-%m-%d') == false)
                $this->endDate = date("Y-m-d");
            else
                $this->endDate = $end;
        }
        else
            $this->endDate = $data['endDate'];
        $dateTmp = range(strtotime($this->startDate), strtotime($this->endDate), 24*60*60);
        if(!empty($dateTmp)) {
            foreach ($dateTmp as $date) {
                $this->dateRange[] = date("Y-m-d", $date);
            }
        }
        return parent::beforeAction($action); // TODO: Change the autogenerated stub
    }

    public function actionList($end){
        if(!is_numeric($end))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data)|| !isset($data['limit'])) {
            $data['limit'] = 200;
        }
        if(!isset($data['page'])) {
            $data['page'] = 0;
        }
        $query = VDeviceLog::find()
            ->andWhere("`DateTime` >= '$this->startDate 00:00:00' and `DateTime` < '$this->endDate 23:59:59' and (`EndTime` <= '$this->endDate 23:59:59' or `EndTime` is null)")
            ->andWhere("DevLogID <= $end");
        return $this->list($query, $data);
    }

    public function actionDownload(){
        $type = Yii::$app->getRequest()->get('type', 'device');
        if($type == 'user') {
            $query = VDeviceLog::find()
                ->select(['User_Sn', 'User_Name', 'DeviceName', 'DeviceID', 'CtrlUser', 'DateTime', 'EndTime', 'Length']);
            $query->andWhere(['not', ['CtrlUser' => null]]);
            $ret =  Yii::t('app/message', 'log/User_Sn') . ',' . Yii::t('app/message', 'log/User_Name') . ',' .
                Yii::t('app/message', 'log/DeviceName') . ',' .
                Yii::t('app/message', 'log/DeviceID') . ',' .Yii::t('app/message', 'log/DateTime') . ',' .
                Yii::t('app/message', 'log/EndTime') . ',' .Yii::t('app/message', 'log/Length') . "\r\n";
            $filename = "userlog-$this->startDate-$this->endDate.csv";
        }
        else {
            $query = VDeviceLog::find()
                ->select(['DeviceID', 'DeviceName', 'User_Name', 'CtrlUser','DateTime', 'EndTime', 'Length', 'Event']);
            $ret = Yii::t('app/message', 'log/DeviceID') . ',' .Yii::t('app/message', 'log/DeviceName') . ',' .
                Yii::t('app/message', 'log/User_Name') . ',' .Yii::t('app/message', 'log/DateTime') . ',' .
                Yii::t('app/message', 'log/EndTime') . ',' .Yii::t('app/message', 'log/Length') . ',' .
                Yii::t('app/message', 'log/Event') ."\r\n";
            $filename = "devicelog-$this->startDate-$this->endDate.csv";
        }
        $query->andWhere("`DateTime` >= '$this->startDate 00:00:00' and `DateTime` < '$this->endDate 23:59:59' and (`EndTime` <= '$this->endDate 23:59:59' or `EndTime` is null)");
        $event = Yii::$app->getRequest()->get('event');
        if(empty($event))
            $query->andWhere(['Event' => [1,2,3,4,5,6]]);
        else {
            $event = explode(',', $event);
            foreach ($event as $ev) {
                if(!is_numeric($ev))
                    return $this->response(Error::STATUS_PARAM_ERROR);

            }
            $query->andWhere(['Event' => $event]);
        }
        $filepath = Common::MEDIA_TEMPORARA_DIR . Yii::$app->security->generateRandomString(16). ".tmp";
        if(file_exists($filepath)) {
            unlink($filepath);
        }
        $fp = fopen($filepath, "a");
        fwrite($fp, $ret);

        $query->asArray();
        $count = $this->listQuery($query);
        for($i = 0; $i < $count / 50000; $i++) {
            $newQuery = $query;
            $newQuery->offset($i * 50000);
            if($count - $i * 50000 >= 50000)
                $newQuery->limit(50000);
            else
                $newQuery->limit($count - $i * 50000);
            $lists = $newQuery->all();
            $tmp = '';
            foreach ($lists as $list) {
                if(is_null($list['User_Name']) && !is_null($list['CtrlUser'])) {
                    $list['User_Name'] = 'UID:' . $list['CtrlUser'];
                }
                unset($list['CtrlUser']);
                if($type != 'user')
                    $list['Event'] = Yii::t('app/message', 'log/event_type_' . $list['Event']);
                $tmp .= implode(',', $list) . "\r\n";
            }
            fwrite($fp, $tmp);
        }
        fclose($fp);
        Yii::$app->response->sendFile($filepath, $filename,  ["mimeType" => "text/csv"]) && unlink($filepath);;
    }

    public function actionClean() {
        $this->logTitle['title1'] = $this->startDate;
        $this->logTitle['title2'] = $this->endDate;

        Yii::$app->db
            ->createCommand("Delete from `DeviceStatistic` where `StatisticDay` >= '$this->startDate' and `StatisticDay` <= '$this->endDate';")
            ->execute();
        $cnt = Yii::$app->db
            ->createCommand("DELETE a.* FROM DeviceLog a
                                    WHERE a.`DateTime` >= '$this->startDate 00:00:00'
                                    AND (a.`EndTime` < '$this->endDate 23:59:59'
                                        OR (a.`EndTime` IS NULL
                                            AND a.`DevLogID` NOT IN (SELECT
                                                *
                                            FROM
                                                (SELECT
                                                    DevLogID
                                                FROM
                                                    DeviceLog
                                                WHERE
                                                    `DateTime` >= '$this->endDate 23:59:59') AS tmp)))")
            ->execute();
        return $this->response(Error::STATUS_SUCCESS, ['rows' => $cnt]);
    }

    public function actionIpcallErr(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data)|| !is_numeric($data['limit'])) {
            $data['limit'] = 200;
        }
        if(!is_numeric($data['page'])) {
            $data['page'] = 0;
        }
        if(!is_numeric($data['end']))
            $data['end'] = 0xffffffff;

        $query = VDeviceLog::find()
            ->Where(['<=', 'DevLogID', $data['end']])
            ->andWhere(['Event' => 6])
            ->andWhere(['EndTime' => null]);
        if(!empty($data['device']) && is_array($data['device']))
            $query->andWhere(['DeviceID' => $data['device']]);

        return $this->list($query, $data);
    }

    static function logCacheSet($event, $caches) {
        $file = Common::DEVICE_LOG_STATISTICS_SALT . $event;
        if(file_exists($file))
            unlink($file);
        $ctx = serialize($caches);
        if(!file_put_contents($file, $ctx))
            unlink($file);
        else
            Yii::$app->cache->set($file, true, 86400);
    }

    static function logCacheGet($start, $end, $event, &$needFetch, &$res, &$caches) {
        $unixs = strtotime($start);
        $unixe = strtotime($end);
        $res = [];
        $caches = [];
        $needFetch = [];

        do {
            $file = Common::DEVICE_LOG_STATISTICS_SALT . $event;
            $valid = Yii::$app->cache->get($file);
            if (!$valid)
                break;
            if (!file_exists($file))
                break;
            $ctx = file_get_contents($file);
            if (!$ctx)
                break;
            $caches = unserialize($ctx);
            if (!$caches)
                $caches = [];
        }while(false);
        $tmp = $unixs;
        $now = date("Y-m-d");
        $fetch = [];
        while($tmp <= $unixe) {
            $k = date("Y-m-d", $tmp);
            if(!array_key_exists($k, $caches) || $k == $now) {
                if(empty($fetch))
                    $fetch['start'] = $k;
            }
            else {
                if(!empty($fetch)) {
                    $end = date("Y-m-d", $tmp - 86400);
                    $fetch['end'] = $end;
                    $needFetch[] = $fetch;
                    $fetch = [];
                }
                $res[] = $caches[$k];
            }
            $tmp += 3600 * 24;
        }
        if(!empty($fetch)) {
            $fetch['end'] = date('Y-m-d', $unixe);
            $needFetch[] = $fetch;
        }
    }

    /**
     * 开机时长统计
     * @return array
     * @throws \yii\db\Exception
     */
    public function actionPowerOnInterval(){
        self::logCacheGet($this->startDate, $this->endDate, $this->action->id, $needFetch, $res, $caches);
        //cache保存的是数据库能查到今天之前的有效日期的数据统计，不存在的日期或者当天的数据需要重新到数据库查询
        if(empty($needFetch))
            return $this->response(Error::STATUS_SUCCESS, $res);
        foreach ($needFetch as $fetch) {
            $sql = "select  DATE_FORMAT(`DateTime`, '%Y-%m-%d') as date, sum(timestampdiff(second, `DateTime`,`EndTime`)) as length from v_device_log
                    where `event` = 2 and `EndTime` is not null  and `DateTime`> '" . $fetch['start'] . " 00:00:00' and `DateTime`<= '" . $fetch['end'] . " 23:59:59' group by date;";
            $items = Yii::$app->db->createCommand($sql)->queryAll();
            foreach ($items as $item) {
                $caches[$item['date']] = $item;
                $res[] = $item;
            }
        }
        //将用户需要的时间段日期与实际查询到的有效的时间段日期比较，如果有效的时间段没有用户需要的日期，则补充。
        $resDate = [];
        foreach ($res as $r)
            $resDate[] = $r['date'];
        foreach ($this->dateRange as $date) {
            if(!in_array($date, $resDate)) {
                $res[] = ['date' => $date, 'length' => 0];
                $caches[$date] = ['date' => $date, 'length' => 0];//无效的日期，保存到cache，以便下次不再需要fetch
            }
        }
        array_multisort(array_column($res,'date'),SORT_ASC,$res);//新的$res按键值‘date’升序排序

        self::logCacheSet($this->action->id, $caches);
        return $this->response(Error::STATUS_SUCCESS, $res);
    }

    /**
     * 离线次数统计
     * @return array
     * @throws \yii\db\Exception
     */
    public function actionOfflineStatistic(){
        self::logCacheGet($this->startDate, $this->endDate, $this->action->id, $needFetch, $res, $caches);
        if(empty($needFetch))
            return $this->response(Error::STATUS_SUCCESS, $res);
        $test = [];
        foreach ($needFetch as $fetch) {
            $sql = "select  DATE_FORMAT(`EndTime`, '%Y-%m-%d') as date, count(*) as count from v_device_log
                    where `event` = 1 and `EndTime`> '" . $fetch['start'] . " 00:00:00' and `EndTime`<= '" . $fetch['end'] . " 23:59:59' group by date;";
            $items = Yii::$app->db->createCommand($sql)->queryAll();
            $c = [];
            $test[] = $items;
            foreach ($items as $item) {
                $c[$item['date']] = ['date' => $item['date'], 'offline' => $item['count'] + 0];
            }
            $caches = array_merge($caches, $c);
            foreach ($c as $i)
                $res[] = $i;
        }

        $resDate = [];
        foreach ($res as $r)
            $resDate[] = $r['date'];
        foreach ($this->dateRange as $date) {
            if (!in_array($date, $resDate)) {
                $res[] = ['date' => $date, 'offline' => 0];
                $caches[$date] = ['date' => $date, 'offline' => 0];
            }
        }
        array_multisort(array_column($res,'date'),SORT_ASC,$res);//新的$res按键值‘date’升序排序

        self::logCacheSet($this->action->id, $caches);
        return $this->response(Error::STATUS_SUCCESS, $res);
    }

    /**
     * 开机次数统计
     * @return array
     * @throws \yii\db\Exception
     */
    public function actionPowerOnStatistic(){
        self::logCacheGet($this->startDate, $this->endDate, $this->action->id, $needFetch, $res, $caches);
        if(empty($needFetch))
            return $this->response(Error::STATUS_SUCCESS, $res);
        $test = [];
        foreach ($needFetch as $fetch) {
            $sql = "select  DATE_FORMAT(`DateTime`, '%Y-%m-%d') as date, `event` as event, count(*) as count from v_device_log
                    where `event` in (2, 11, 12) and `DateTime`> '" . $fetch['start'] . " 00:00:00' and `DateTime`<= '" . $fetch['end'] . " 23:59:59' group by date, event;";
            $items = Yii::$app->db->createCommand($sql)->queryAll();
            $c = [];
            $test[] = $items;
            foreach ($items as $item) {
                if(empty($c[$item['date']]))
                    $c[$item['date']] = ['date' => $item['date'], 'on' => 0, 'ic_success' => 0, 'ic_fail' => 0];
                if($item['event'] == 2)
                    $c[$item['date']]['on'] = $item['count'] + 0;
                else if($item['event'] == 11)
                    $c[$item['date']]['ic_success'] = $item['count'] + 0;
                else if ($item['event'] == 12)
                    $c[$item['date']]['ic_fail'] = $item['count'] + 0;
            }
            $caches = array_merge($caches, $c);
            foreach ($c as $i)
                $res[] = $i;
        }
        $resDate = [];
        foreach ($res as $r)
            $resDate[] = $r['date'];
        foreach ($this->dateRange as $date) {
            if (!in_array($date, $resDate)) {
                $res[] = ['date' => $date, 'on' => 0, 'ic_success' => 0, 'ic_fail' => 0];
                $caches[$date] = ['date' => $date, 'on' => 0, 'ic_success' => 0, 'ic_fail' => 0];
            }
        }
        array_multisort(array_column($res,'date'),SORT_ASC,$res);//新的$res按键值‘date’升序排序

        self::logCacheSet($this->action->id, $caches);
        return $this->response(Error::STATUS_SUCCESS, $res);
    }

    /**
     * 能耗统计
     * @return array
     * @throws \yii\db\Exception
     */
    public function actionEnergy(){
        self::logCacheGet($this->startDate, $this->endDate, $this->action->id, $needFetch, $res, $caches);
        if(empty($needFetch))
            return $this->response(Error::STATUS_SUCCESS, $res);

        $cols = [];
        for($i = 0; $i < 17; $i++) {
            $cols[] = "sum(power$i) as power$i";
        }

        $cols = implode(',', $cols);
        foreach ($needFetch as $fetch) {
            $sql = "select StatisticDay, $cols from v_device_statistic where StatisticDay >= '" . $fetch['start'] . "' and StatisticDay <= '" . $fetch['end'] . "' group by StatisticDay";
            $rows = Yii::$app->db->createCommand($sql)->queryAll();
            foreach ($rows as $row) {
                $caches[$row['StatisticDay']] = $row;
                $res[]= $row;
            }
        }

        if(!empty($rows))
            self::logCacheSet($this->action->id, $caches);

        return $this->response(Error::STATUS_SUCCESS, $res);
    }
}