<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/03/08 16:37
 */
namespace api\controllers;
use common\classes\Common;
use common\components\Media;
use common\models\MediaFiles;
use common\models\Users;
use common\models\VMediaFiles;
use common\models\VSessionFiles;
use common\models\BaseInfo;
use Yii;
use common\models\UploadFile;
use common\classes\Error;
use yii\db\Exception;
use yii\db\mssql\PDO;
use yii\filters\VerbFilter;
use yii\web\UploadedFile;

/**
 * Session controller
 */
class MediaController extends BaseController
{
    public $checkAction = ['check-file'];
    public $hasFileCheckPri = false;
    public $needFileCheckPri = false;
    public function beforeAction($action)
    {
        $ret = parent::beforeAction($action);
        if (!$ret) {
            return $ret;
        }
        $model = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_UPLOAD_FILES_CONFIG]);
        if (!empty($model) && $model->InfoValue === 1)
            $this->needFileCheckPri = true;
        if(in_array('fcheck', $this->userinfo['pri']) || $this->isSuper || !$this->needFileCheckPri)
            $this->hasFileCheckPri = true;
        if(in_array($action->id, $this->checkAction) && !$this->hasFileCheckPri) {
            Yii::$app->response->data = $this->response(Error::STATUS_PRIVILEGE_ERROR);
            return false;
        }
        return $ret;
    }
    public function actionFolderList(){
        if($this->isSuper)
            $subFolder = VMediaFiles::find()->select(['MediaID', 'FileName', 'Parent', 'Owner', 'Sysfolder'])
                ->where(['AudioCodec' => 0, 'VedioCodec' => 0])
                ->all();
        else {
            $suser = Users::find()
                ->select(['User_ID'])
                ->where('privilege & 0x140 = 0x140')
                ->all();
            $uids = [];
            foreach ($suser as $uid) {
                $uids[] = $uid->User_ID;
            }
            $uids[] = (Yii::$app->user->id + 0);
            $media = VMediaFiles::find()
                ->select(['MediaID', 'FileName', 'Parent', 'Owner', 'Sysfolder'])
                ->where(['Owner' => $uids, 'Sysfolder' => 1])
                ->all();
            if(empty($media)) {
                $media = new MediaFiles();
                $media->FileName = Yii::$app->user->getIdentity()->User_Desc;
                $media->Parent = 1;
                $media->Owner = Yii::$app->user->getId();
                $media->Sysfolder = 1;
                $media->save();
                $subFolder = [$media];
            }
            else
                $subFolder = $media;
            $parent = [];
            foreach ($subFolder as $f)
                $parent[] = $f->MediaID;
            while(true){
                $temp = VMediaFiles::find()
                    ->select(['MediaID', 'FileName', 'Parent'])
                    ->where(['VedioCodec' => 0, 'AudioCodec' => 0])
                    ->andWhere(['in', 'parent', $parent])
                    ->indexBy('MediaID')
                    ->all();
                if(empty($temp))
                    break;
                $parent = [];
                foreach ($temp as $index => $value) {
                    $subFolder[] = $value;
                    $parent[] = $index;
                }
            }
            $subFolder[] = VMediaFiles::find()
                ->select(['MediaID', 'FileName', 'Parent', 'Owner', 'Sysfolder'])
                ->where(['Sysfolder' => 2])
                ->one();
        }
        return $this->response(Error::STATUS_SUCCESS, $subFolder);
    }

    private function folderView($id) {
        return VMediaFiles::find()->select(['MediaID', 'FileName', 'Parent'])
            ->where(['AudioCodec' => 0, 'VedioCodec' => 0, 'MediaID' => $id])
            ->one();
    }

    public function actionFolderNew(){
        $data = Yii::$app->request->post('data');
        if(is_null($data) || !isset($data['FileName']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $folder = new MediaFiles();
        $folder->FileName = $data['FileName'];

        if(!isset($data['Parent']))
            $folder->Parent = $this->isSuper ? null: Yii::$app->user->getId();
        else {
            $own = Yii::$app->db->createCommand('select func_get_mediafile_Owner('. $data['Parent'] . ')')
                ->queryScalar();
            if($own != Yii::$app->user->getId() && !$this->isSuper)
                return $this->response(Error::STATUS_FORBID);
            $parentFolder = VMediaFiles::findOne(['MediaID' => $data['Parent']]);
            if(is_null($parentFolder))
                return $this->response(Error::STATUS_FOLDER_PARENT_NOT_EXISTS);
            if($parentFolder->Sysfolder == 2)
                return $this->response(Error::STATUS_FORBID);
            $folder->Parent = $data['Parent'];
        }
        if(!$folder->save())
            return $this->response(Error::STATUS_FAILED);
        return $this->response(Error::STATUS_SUCCESS, $this->folderView($folder->MediaID));
    }

    public function actionFolderUpdate($id){
        $data = Yii::$app->request->post('data');
        if(is_null($data) || empty($data['FileName']) || !preg_match('/^[0-9][0-9,]*$/', $id))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $own = Yii::$app->db->createCommand('select func_get_mediafile_Owner('. $id . ')')->queryScalar();
        if($own != Yii::$app->user->getId() && !$this->isSuper)
            return $this->response(Error::STATUS_FORBID);

        $folder = MediaFiles::findOne(['MediaID' => $id]);
        if(is_null($folder))
            return $this->response(Error::STATUS_FOLDER_NOT_EXISTS);
        if(isset($folder->Sysfolder))
            return $this->response(Error::STATUS_FORBID);
        $folder->FileName = $data['FileName'];
        if(!isset($folder->owner)) {
            if (!isset($data['Parent']))
                $folder->Parent = $this->isSuper ? null : Yii::$app->user->getId();
            else {
                $parentFolder = VMediaFiles::findOne(['MediaID' => $data['Parent']]);
                if(is_null($parentFolder))
                    return $this->response(Error::STATUS_FOLDER_PARENT_NOT_EXISTS);
                if($parentFolder->Sysfolder == 2)
                    return $this->response(Error::STATUS_FORBID);
                $folder->Parent = $data['Parent'];
            }
        }
        if(!$folder->save())
            return $this->response(Error::STATUS_FAILED);
        return $this->response(Error::STATUS_SUCCESS, $this->folderView($folder->MediaID));
    }

    public function actionLite($type, $folder){
        $data = Yii::$app->request->post('data');
        $query = VMediaFiles::find()
            ->select(['MediaID', 'FileName', 'TotalTime','AudioCodec', 'VedioCodec'])
            ->where(['or', ['Sysfolder' => null], 'Sysfolder >= -1'])
            ->andWhere(['`Check`' => 0]);
        if($type == 'audio')
            $query = $query->andWhere(['VedioCodec' => -1, 'AudioCodec' => [0,1,2,3]]);
        else if($type == 'video')
            $query = $query->andWhere(['or', ['VedioCodec' => [0,1,2], 'AudioCodec' =>[-1,1,2,3]], ['VedioCodec' => -1, 'AudioCodec' => -1]]);
        else {
            $query = $query->andWhere(['or', ['VedioCodec' => -1, 'AudioCodec' => [-1,0,1,2,3]],['VedioCodec' => [0,1,2], 'AudioCodec' =>[-1,1,2,3]]]);
        }
        if($folder == "true")
            $query = $query->orWhere(['VedioCodec' => 0, 'AudioCodec' => 0]);
        return $this->list($query, $data);
    }

    public function actionList($type, $folder){
        $data = Yii::$app->request->post('data');
        $query = VMediaFiles::find()
            ->where(['or', ['Sysfolder' => null], 'Sysfolder >= 0'])
            ->andWhere(['`Check`' => 0]);
        if($type == 'audio')
            $query = $query->andWhere(['VedioCodec' => -1, 'AudioCodec' => [0,1,2,3]]);
        else if($type == 'video')
            $query = $query->andWhere(['or', ['VedioCodec' => [0,1,2], 'AudioCodec' =>[-1,1,2,3]], ['VedioCodec' => -1, 'AudioCodec' => -1]]);
        else {
            $query = $query->andWhere(['or', ['VedioCodec' => -1, 'AudioCodec' => [-1,0,1,2,3]],['VedioCodec' => [0,1,2], 'AudioCodec' =>[-1,1,2,3]]]);
        }
        if($folder == "true")
            $query = $query->orWhere(['VedioCodec' => 0, 'AudioCodec' => 0]);
        return $this->list($query, $data);
    }

    /**
     * get file detail info
     * @param $id file id
     * @return array
     */
    public function actionView($id){
        $file = MediaFiles::findOne(['MediaID' => $id]);
        if(is_null($file))
            return $this->response(Error::STATUS_MEDIA_FILE_NOT_EXISTS);
        $sessions = VSessionFiles::find()
            ->select(['SessionName'])
            ->where(['FileID' => $file->MediaID])
            ->all();
        return $this->response(Error::STATUS_SUCCESS, [
            'file' => $file,
            'sessions' => $sessions
        ]);
    }

    /**
     * @param $id
     * @return array
     */
    public function actionUpdate($id){
        $data = Yii::$app->request->post('data');
        if(is_null($data) || !preg_match('/^[0-9][0-9,]*$/', $id))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $own = Yii::$app->db->createCommand('select func_get_mediafile_Owner('. $id . ')')->queryScalar();
        if($own != Yii::$app->user->getId() && !$this->isSuper)
            return $this->response(Error::STATUS_FORBID);

        $file = MediaFiles::findOne(['MediaID' => $id]);
        if(is_null($file))
            return $this->response(Error::STATUS_MEDIA_FILE_NOT_EXISTS);
        if(isset($file->Sysfolder))
            return $this->response(Error::STATUS_FORBID);
        if($file->TotalTime == 10 && $file->FileFmt == -1) {
            if(empty($data['FilePath']))
                return $this->response(Error::STATUS_PARAM_ERROR);
            else
                $file->FilePath = $data['FilePath'];
        }
        if(!empty($data['FileName']))
            $file->FileName = $data['FileName'];
        $this->logTitle['title1'] = $file->FileName;
        if(isset($data['Gain'])) {
            if($data['Gain'] > 10)
                $file->Gain = 10;
            else if($data['Gain'] < -10)
                $file->Gain = -10;
            else
                $file->Gain = $data['Gain'];
        }
        if(isset($data['Parent'])) {
            $parentFolder = VMediaFiles::findOne(['MediaID' => $data['Parent']]);
            if (is_null($parentFolder))
                return $this->response(Error::STATUS_FOLDER_PARENT_NOT_EXISTS);
            if ($parentFolder->Sysfolder == 2)
                return $this->response(Error::STATUS_FORBID);
            $file->Parent = $data['Parent'];
        }
        $Check = $file->Check;
        if($file->load(Yii::$app->request->post(), 'data')){
            $file->Check = $Check;
            if($file->save())
                return $this->response(Error::STATUS_SUCCESS, $this->folderView($file->MediaID));
        }
        return $this->response(Error::STATUS_FAILED);
    }

    public function actionNew(){
        $data = Yii::$app->request->post('data');
        if(is_null($data) || !isset($data['FileName'], $data['Gain'], $data['FilePath']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        if(isset($data['Parent'])) {
            $parentFolder = MediaFiles::findOne(['MediaID' => $data['Parent'], 'AudioCodec' => 0, 'VedioCodec' => 0]);
            if(empty($parentFolder))
                return $this->response(Error::STATUS_FOLDER_PARENT_NOT_EXISTS);
            if(!$this->isSuper){
                $tryCnt = 20;
                while ($parentFolder->Owner != Yii::$app->user->id) {
                    $parentFolder = MediaFiles::findOne(['MediaID' => $parentFolder->Parent, 'AudioCodec' => 0, 'VedioCodec' => 0]);
                    $tryCnt--;
                    if(empty($parentFolder) || is_null($parentFolder->Parent) || $tryCnt <= 0)
                        return $this->response(Error::STATUS_FOLDER_PARENT_NOT_EXISTS);
                }
            }
        }
        else if(!$this->isSuper)
            return $this->response(Error::STATUS_FORBID);
        else
            $parent = null;
        $file = new MediaFiles();
        $file->FileName = $data['FileName'];
        $file->Gain = $data['Gain'];
        $file->FilePath = $data['FilePath'];
        $file->Parent = $data['Parent'];
        if(!$this->isSuper)
            $file->Parent = isset($data['Parent'])? $data['Parent'] : Yii::$app->user->getId();
        $file->TotalTime = 10;
        $file->FileFmt = -1;
        $file->VedioCodec = -1;
        $file->AudioCodec = -1;
        $this->logTitle['title1'] = $file->FileName;
        if($file->save())
            return $this->response(Error::STATUS_SUCCESS, $file);
        return $this->response(Error::STATUS_FAILED);
    }

    public function actionDelete($id){
        if(!preg_match('/^[0-9,]*$/', $id))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $ids = explode(',', $id);
        if(empty($ids))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $mediaid = [];
        foreach ($ids as $id) {
            if(is_numeric($id))
                $mediaid[] = $id;
        }
        if(empty($mediaid))
            return $this->response(Error::STATUS_SUCCESS, $mediaid);
        $uid = Yii::$app->user->getId();
        if($this->isSuper)
            $uid = 0;
        $id = 0;
        $delCmd = Yii::$app->db->createCommand('call sp_del_media_file(:uid, :mediaid);')
            ->bindParam(':uid', $uid, PDO::PARAM_INT)
            ->bindParam(':mediaid', $id, PDO::PARAM_INT);

        $ids = implode(',', $mediaid);
        $files = [];
        $lists = Yii::$app->db->createCommand("select MediaID, func_get_mediafile_Owner(MediaID) as Owner, `Check`,`FileName` from MediaFiles where MediaID in ($ids)")
            ->queryAll();
        foreach ($lists as $list) {
            $files[] = $list['FileName'];
            if($list['Check'] == 1){
                if($list['Owner'] != $uid && !$this->hasFileCheckPri)
                    continue;
                $uid = $list['Owner'];
            }else {
                $fileNum = MediaFiles::find()->where(['Parent'=>$list['MediaID']])->count();
                if($fileNum > 0)
                    continue;
                if ($list['Owner'] != $uid && !$this->isSuper)
                    continue;
            }
            $id = $list['MediaID'];
            $delCmd->execute();
        }
        $title1 = implode(",", $files);
        if(mb_strlen($title1) > 256)
            $this->logDetail['ele/files'] = mb_substr($title1, 0, 256) . "...";
        else
            $this->logDetail['ele/files'] = $title1;
        if(mb_strlen($title1) > 64)
            $title1 = mb_substr($title1, 0, 64) . "...";
        $this->logTitle['title1'] = $title1;
        //驳回并删除文件日志
        $data = Yii::$app->getRequest()->post('data');
        if(array_key_exists('reason', $data) && !empty($data['reason']) && $data['reason'] == 'reject') {
            $this->writeLog(Error::STATUS_SUCCESS, $this->logDetail, 'reject');
            $this->logSended = true;
        }

        $items = MediaFiles::find()->select(['MediaID'])->where(['in', 'MediaID', $mediaid])->all();
        $ret = [];
        foreach ($items as $item) {
            $ret[] = $item->MediaID;
        }
        if(!empty($ret))
            $ret = ['failed' => $ret];
        return $this->response(Error::STATUS_SUCCESS, $ret);
    }

    public function actionUpload($parent)
    {
        if(is_numeric($parent)) {
            $parentFolder = MediaFiles::findOne(['MediaID' => $parent, 'AudioCodec' => 0, 'VedioCodec' => 0]);
            if(empty($parentFolder))
                return $this->response(Error::STATUS_FOLDER_PARENT_NOT_EXISTS);
            if(!$this->isSuper){
                $tryCnt = 20;
                while ($parentFolder->Owner != Yii::$app->user->id) {
                    $parentFolder = MediaFiles::findOne(['MediaID' => $parentFolder->Parent, 'AudioCodec' => 0, 'VedioCodec' => 0]);
                    $tryCnt--;
                    if(empty($parentFolder) || is_null($parentFolder->Parent) || $tryCnt <= 0)
                        return $this->response(Error::STATUS_FOLDER_PARENT_NOT_EXISTS);
                }
            }
        }
        else if($parent == 'null' && !$this->isSuper)
            return $this->response(Error::STATUS_FORBID);
        else
            $parent = null;
        $model = new UploadFile();
        $model->multiple = UploadedFile::getInstancesByName('multi-file');
        if(empty($model->multiple)) {
            return $this->response(Error::STATUS_FAILED);
        }
        $files = $model->uploadMultiple($parent);
        $names = [];
        $mids = [];
        $ret = Error::STATUS_FAILED;
        if(!empty($files)) {
            foreach ($files as $key => $value) {
                if($value['error'] != Error::STATUS_SUCCESS) {
                    $ret = $value['error'];
                    continue;
                }
                $names[] = $key;
                $mids[] = $value['media']['MediaID'];
            }
        }
        if(!count($mids)) {
            $this->logSended = true;
            return $this->response($ret);
        }
        $this->logDetail['ele/files']  = implode("\n", $names);

        $title1 = implode(",", $names);
        $username = empty(Yii::$app->user->getIdentity()->User_Desc) ? Yii::$app->user->getIdentity()->User_Name : Yii::$app->user->getIdentity()->User_Desc;
        if($this->isSuper || in_array('fcheck', $this->userinfo['pri'])) {
            if (mb_strlen($title1) > 10)
                Yii::$app->udp->mediaUploadMessage(date('Y-m-d h:i:s'), mb_substr($title1, 0, 8) . "...", $username);
            else
                Yii::$app->udp->mediaUploadMessage(date('Y-m-d h:i:s'), $title1, $username);
        }
        else {
            if (mb_strlen($title1) > 10)
                Yii::$app->udp->mediaCheckMessage(date('Y-m-d h:i:s'), mb_substr($title1, 0, 8) . "...", $username);
            else
                Yii::$app->udp->mediaCheckMessage(date('Y-m-d h:i:s'), $title1, $username);
        }
        if(mb_strlen($title1) > 64)
            $title1 = mb_substr($title1, 0, 64) . "...";
        $this->logTitle['title1'] = $title1;

        $mid = implode(',', $mids);
        Yii::$app->db->createCommand('update MediaFiles set Owner = '. Yii::$app->user->id . ' where MediaID in (' . $mid .');')
            ->execute();
        if(!$this->hasFileCheckPri) {
            Yii::$app->db->createCommand('update MediaFiles set `Check` = 1' . ' where MediaID in (' . $mid .');')
                ->execute();
            $this->writeLog(Error::STATUS_SUCCESS, $this->logDetail, 'upload_check');
            $this->logSended = true;
            return $this->response(Error::STATUS_FILE_NEED_CHECK);
        }
        else
            return $this->response(Error::STATUS_SUCCESS, ['files' => $files]);
    }

    public function actionMove(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !isset($data['MediaID']) || !preg_match('/^[0-9][0-9,]*$/', $data['MediaID']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        if(isset($data['Parent'])) {
            $parent = MediaFiles::findOne(['MediaID' => $data['Parent']]);
            if(empty($parent))
                return $this->response(Error::STATUS_FOLDER_NOT_EXISTS);
            if($parent->VedioCodec != 0 || $parent->AudioCodec != 0)
                return $this->response(Error::STATUS_FILE_TYPE_ERROR);
            $this->logDetail['ele/target'] = $parent->FileName;
        }
        else {
            $this->logDetail['ele/target'] = "根目录";
            $data['Parent'] = 'null';
        }

        $mid = explode(',', $data['MediaID']);
        $owns = Yii::$app->db->createCommand('select func_get_mediafile_Owner('. $mid[0] . ')')->queryScalar();
        $ownd = Yii::$app->db->createCommand('select func_get_mediafile_Owner('. $data['Parent'] . ')')->queryScalar();
        if(!$this->isSuper && ($owns != Yii::$app->user->getId() || $ownd != Yii::$app->user->getId()))
            return $this->response(Error::STATUS_FORBID);

        $files = MediaFiles::find()->select(['FileName'])->where('MediaID in (' . $data['MediaID'] .')')
            ->all();
        $names = [];
        foreach ($files as $file) {
            $names[] = $file->FileName;
        }
        $title1 = implode(",", $names);
        if(mb_strlen($title1) > 256)
            $this->logDetail['ele/files'] = mb_substr($title1, 0, 256) . "...";
        else
            $this->logDetail['ele/files'] = $title1;
        if(mb_strlen($title1) > 64)
            $title1 = mb_substr($title1, 0, 64) . "...";
        $this->logTitle['title1'] = $title1;
        if(empty($this->logDetail['ele/files'])) {
            $this->logSended = true;
            $this->response(Error::STATUS_FAILED);
        }
        else {
            Yii::$app->db->createCommand('update MediaFiles set Parent=' . $data['Parent'] . ' where MediaID in (' . $data['MediaID'] .');')
                ->execute();
            return $this->response(Error::STATUS_SUCCESS);
        }
    }

    public function actionCheckFileList(){
        $data = Yii::$app->request->post('data');
        if(empty($data) || !isset($data['onlynum']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $query = VMediaFiles::find()
            ->Where(['`Check`' => 1])
            ->andwhere(['or', ['VedioCodec' => [-1,1,2]], ['AudioCodec' => [-1,1,2,3]]])
            ->andWhere(['Sysfolder' => null]);

        if(!$this->hasFileCheckPri)
            $query->andWhere(['Owner' => Yii::$app->user->getId()]);

        if($data['onlynum'] == 1) {
            $count = $this->listQuery($query);
            return $this->response(Error::STATUS_SUCCESS, ['count' => $count]);
        }
        else
            return $this->list($query, $data);

    }

    public function actionCheckFile(){
        $data = Yii::$app->request->post('data');
        if(empty($data) || !preg_match('/^[0-9][0-9,]*$/', $data['MediaID']))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $mids = explode(',', $data['MediaID']);
        $names = [];
        foreach ($mids as $mid){
            $file = MediaFiles::findOne(['MediaID' => $mid]);
            if(empty($file))
                continue;
            $names[] = $file->FileName;
        }

        Yii::$app->db->createCommand('update MediaFiles set `Check` = 0' . ' where MediaID in (' . $data['MediaID'] .');')
            ->execute();

        $title1 = implode(",", $names);
        if(mb_strlen($title1) > 256)
            $this->logDetail['ele/files'] = mb_substr($title1, 0, 256) . "...";
        else
            $this->logDetail['ele/files'] = $title1;
        if(mb_strlen($title1) > 64)
            $title1 = mb_substr($title1, 0, 64) . "...";
        $this->logTitle['title1'] = $title1;

        return $this->response(Error::STATUS_SUCCESS);
    }
}