<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/03/08 16:37
 */
namespace api\controllers;

use common\models\AlternateDate;
use common\models\BroadcastMode;
use common\models\DeviceList;
use common\models\Players;
use common\models\PlaySessionTime;
use common\models\TaskTemplate;
use common\models\VDevice;
use common\models\VPlayerKeys;
use common\models\VPlayingSessions;
use Yii;
use yii\db\Exception;
use common\classes\Error;
use common\models\VSessionInfo;
use common\models\VTimedSessionInfo;
use common\models\VSession;
use common\models\BaseInfo;
use common\models\VSessionFiles;
use common\classes\Common;
use common\models\PlaySessions;
use yii\helpers\ArrayHelper;
use yii\helpers\Json;

/**
 * Session controller
 */
class SessionController extends SessionBaseController
{
    public $menualAction = ['manual-list', 'manual-list-lite'];
    public $realtimeAction = ['realtime-list', 'realtime-ctrl', 'realtime-media-list', 'realtime-view','realtime-update','realtime-delete', 'player-ctrl'];
    public $optionalAction = ['player-status'];
    public function behaviors()
    {
        return  ArrayHelper::merge (
            parent::behaviors(),[
            'authenticator' => [
                'optional' => $this->optionalAction
            ]
        ]);
    }

    public function beforeAction($action)
    {
        $ret = parent::beforeAction($action); // TODO: Change the autogenerated stub
        if (!$ret || in_array($action->id, $this->optionalAction)) {
            return $ret;
        }
        if (!$this->isSuper && in_array($action->id, $this->menualAction) && !in_array('manual', $this->userinfo['pri'])) {
            Yii::$app->response->data = $this->response(Error::STATUS_PRIVILEGE_ERROR);
            return false;
        }
        if (!$this->isSuper && in_array($action->id, $this->realtimeAction) && !in_array('realtime', $this->userinfo['pri'])) {
            Yii::$app->response->data = $this->response(Error::STATUS_PRIVILEGE_ERROR);
            return false;
        }
        return $ret;
    }

    /**
     * get timed tasks
     */
    public function actionTimedList(){
        $template = Yii::$app->base->activeTemplateId;
        $data = Yii::$app->getRequest()->post('data');
        $query = VTimedSessionInfo::find()
            ->where(['or', ['Template' => $template], ['Template' => null]])
            ->andWhere(['BrdMode' => null]);
        if(!$this->isSuper)
            $query = $query->andWhere(['Owner' => Yii::$app->getUser()->getId()]);
        $query = $query->orderBy(['PlayTime' => SORT_ASC]);
        return $this->getSessionList($query, $data);
    }

    /**
     * get manual tasks
     */
    public function actionManualList()
    {
        $data = Yii::$app->getRequest()->post('data');
        $query = VSessionInfo::find()->where(['SessionType' => [2, 9]])
            ->orderBy(['SessionName' => SORT_ASC]);
        if (!$this->isSuper)
            $query = $query->andWhere(['Owner' => Yii::$app->getUser()->getId()]);
        return $this->getSessionList($query, $data);
    }

    /**
     * get manual tasks
     */
    public function actionManualListLite()
    {
        $data = Yii::$app->getRequest()->post('data');
        $query = VSessionInfo::find()->select(['SessionID','SessionName'])->where(['SessionType' => [2, 9]])
            ->orderBy(['SessionName' => SORT_ASC]);
        if (!$this->isSuper)
            $query = $query->andWhere(['Owner' => Yii::$app->getUser()->getId()]);
        if (($this->serverPri & 0x06) == 0)
            $query = $query->andWhere(['Type' => 'ctrl']);
        else if(($this->serverPri & 0x04) == 0)
            $query = $query->andWhere(['or', ['StreamType' => null], ['StreamType' => [1, 129]]]);
        $list = $query->all();
        return $this->response(Error::STATUS_SUCCESS, $list);
    }

    /**
     * get realtime task
     */
    public function actionRealtimeList(){
        $lists1 = Yii::$app->db->createCommand('select a.PlayerID, a.SessionID as RealSessionID, a.SessionName, a.Priority, a.AudioGain, a.Owner, a.OwnerName, a.StreamType, a.SessionType,
                                                    c.SessionID, c.ScheduleID, a.PlayerAddress as IpAddress, a.PlayType, a.PlayerType from v_session_info a
                                                    left join PlayerStatus c on c.SessionID  = a.SessionID
                                                    where a.PlayerID is not null;')
            ->queryAll();
        $lists2 = Yii::$app->db->createCommand("select a.id as PlayerID, a.actualSession as RealSessionID, a.name as SessionName, a.ModePriority as Priority, a.audioVol as AudioGain, NULL as Owner, NULL as OwnerName,
                                                    1 as StreamType, NULL as SessionType,
                                                    NULL as SessionID, NULL as ScheduleID, a.ipaddress as IpAddress, NULL as PlayType, NULL as PlayerType from v_device a
                                                    where a.id like '49%';")
            ->queryAll();
        $lists = array_merge($lists1, $lists2);
        foreach ($lists as $key => $list) {
            $lists[$key]['RealSessionID'] = intval($list['RealSessionID']);
            $lists[$key]['AudioGain'] = intval($list['AudioGain']);
            $lists[$key]['PlayType'] = intval($list['PlayType']);
            $lists[$key]['PlayerType'] = intval($list['PlayerType']);
            $lists[$key]['StreamType'] = intval($list['StreamType']);
            $lists[$key]['SessionType'] = intval($list['SessionType']);
            $lists[$key]['SessionID'] = intval($list['SessionID']);
        }
        return $this->response(Error::STATUS_SUCCESS, ['count' => count($lists), 'list' => $lists]);
    }

    /**
     * ctrl realtime state
     */
    public function actionRealtimeCtrl($playerid, $state){
        $session = VSessionInfo::find()->select([
            'SessionID', 'SessionName', 'PlayerID', 'StreamType','PlayerAddress',
            'PlayType', 'PlayerType'])
            ->where(['PlayerID' => $playerid])
            ->one();
        if(empty($session))
            return $this->response(Error::STATUS_PLAYER_NOT_EXISTS);

        $this->logTitle['title1'] = $session->SessionName;
        $this->logTitle['title2'] = $state;

        if(empty($session->PlayerAddress))
            return $this->response(Error::STATUS_PLAYER_OFFLINE);
        if($state == 'stop')
            Yii::$app->db->createCommand('call sp_session_ctrl(' . $session->SessionID. ', 0, null);')
                ->execute();
        else if ($state == 'start')
            Yii::$app->db->createCommand('call sp_session_ctrl(' . $session->SessionID. ', 1, null);')
                ->execute();
        else if ($state == 'pause')
            Yii::$app->db->createCommand('call sp_session_ctrl(' . $session->SessionID. ', 2, null);')
                ->execute();
        return $this->response(Error::STATUS_SUCCESS);
    }

    public function actionRealtimeMediaList() {
        $playerid = Yii::$app->request->post('id');
        $session = VSessionInfo::find()->select([
            'SessionID', 'SessionName', 'SessionType',
            'Priority', 'PlayerID', 'StreamType',
            'OwnerName', 'AudioGain', 'PlayerAddress',
            'PlayType', 'PlayerType'])
            ->where(['PlayerID' => $playerid])
            ->one();
        if(is_null($session))
            return $this->response(Error::STATUS_PLAYER_NOT_EXISTS);

    }

    private function getPlayerMusicFiles($ip) {
        $url = "http://webApp:webApp@" . $ip . "/cgi-bin/getmusic.cgi";
        $ch = curl_init();
        curl_setopt ( $ch, CURLOPT_URL, $url );
        curl_setopt ( $ch, CURLOPT_POST, true );
        curl_setopt ( $ch, CURLOPT_POSTFIELDS, '' );
        curl_setopt ( $ch, CURLOPT_RETURNTRANSFER, 1 );
        curl_setopt ( $ch, CURLOPT_CONNECTTIMEOUT, 15 );
        curl_setopt ( $ch, CURLOPT_HEADER, false );
        $mediaList = curl_exec ( $ch );
        curl_close ( $ch );
        $mediaList = str_replace(["\n", ",}", ',]'], ['', '}', ']'], $mediaList);
        if (!empty($mediaList))
            return json_decode($mediaList);
        else
            return NULL;
    }

    private function getBc01View($playerid) {
        $errno = $this->getPlayerIp($playerid, $ip);
        if($errno)
            return $this->response($errno);
        return $this->response(Error::STATUS_SUCCESS, ['files' => $this->getPlayerMusicFiles($ip)]);
    }
    /**
     * get real time player details
     * @param $playerid
     * @return array
     */
    public function actionRealtimeView($playerid){
        if(substr($playerid, 0, 2) == "49")
            return $this->getBc01View($playerid);
        $session = VSessionInfo::find()->select([
            'SessionID', 'SessionName', 'SessionType',
            'Priority', 'PlayerID', 'StreamType', 'Owner',
            'OwnerName', 'AudioGain', 'PlayerAddress',
            'PlayType', 'PlayerType'])
            ->where(['PlayerID' => $playerid])
            ->one();

        if(is_null($session))
            return $this->response(Error::STATUS_PLAYER_NOT_EXISTS);
        $ret = ['player' => $session];

        if($session->SessionType == 1 && is_null($session->Owner)){
            // Media List
            $errno = $this->getPlayerIp($playerid, $ip);
            if($errno)
                return $this->response($errno);
            $ret['files'] = $this->getPlayerMusicFiles($ip);
            if(is_null($session->Owner)) {
                $ret['devices'] = Yii::$app->getDb()
                    ->createCommand("select group_concat(device) as devices from (
                                        select a.DevID as device from DeviceList a, PlayerKeys b
                                         where b.GroupDisable = 0 and b.GroupID is not null and
                                         sp_dev_belong_group(a.DevID, b.GroupID) = true and
                                          b.PlayerID = '" . $playerid . "'
                                          group by a.DevID) c;")
                    ->queryOne()['devices'];
            }
            $ret['keymap'] = VPlayerKeys::find()->select(['Index', 'GroupID', 'GroupName'])
                ->where(['PlayerID' => $playerid])
                ->orderBy(['Index' => SORT_ASC])
                ->all();
        }

        if($session->SessionType == 10) {
            $ret['playtime'] = Yii::$app->db->createCommand("SELECT CAST(`a`.`PlayTime` AS TIME) AS `PlayTime`,
                    GROUP_CONCAT(`a`.`WeekDay` SEPARATOR ',') AS `WeekDay`,
                    GROUP_CONCAT(`a`.`TempFlag` SEPARATOR ',') AS `TempFlag`,
                    GROUP_CONCAT(`a`.`ScheduleID` SEPARATOR ',') AS `ScheduleID`,
                        a.Length as Length
                    FROM
                        `PlaySessionTime` `a`
                    WHERE `a`.SessionID = " . $session->SessionID . "
                    GROUP BY `a`.`PlayTime`;")
                ->queryAll();
            $temp = Yii::$app->db
                ->createCommand("SELECT group_concat(DeviceID) as devices FROM SessionDevice WHERE SessionID = $session->SessionID")
                ->queryOne();
            if (!empty($temp))
                $ret['devices'] = $temp['devices'];
        }
        return $this->response(Error::STATUS_SUCCESS, $ret);
    }

    /**
     * get realtime task
     */
    public function actionRealtimeUpdate($playerid){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !isset($data['player']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $player = PlaySessions::findOne(['PlayerID' => $playerid]);
        if(is_null($player))
            return $this->response(Error::STATUS_SESSION_NOT_EXISTS);
        $this->logTitle['title1'] = $player->SessionName;
        unset($data['player']['SessionType']);
        if(isset($data['player']['StreamType']))
            $data['player']['StreamType'] = ($data['player']['StreamType'] & 0x80) + ($player->StreamType & 0x7f);
        if($player->load($data, 'player') && $player->save()) {
            $db = Yii::$app->db;
            $updateIdxs = [];
            if($player->SessionType == 1 && isset($data['keymap']) && is_array($data['keymap'])) {
                foreach ($data['keymap'] as $keymap) {
                    if(isset($keymap['Index'])  && is_numeric($keymap['Index'])) {
                        if(!isset($keymap['GroupID']))
                            $group = null;
                        else
                            $group = $keymap['GroupID'];
                        try{
                            $db->createCommand("insert into `PlayerKeys` (`PlayerID`, `Index`, `GroupID`) values (:id, :index, :group) on duplicate key update `GroupID`=VALUES(`GroupID`);",
                                [
                                    ':id' => $playerid, ':index' => $keymap['Index'], ':group' => $group
                                ])->execute();
                            $updateIdxs[] = $keymap['Index'];
                        }catch (Exception $e){
                            return $e;
                        }
                    }
                }
            }
            try {
                if(empty($updateIdxs))
                    $cmd = $db->createCommand('update `PlayerKeys` set `GroupID`=null where  `PlayerID`=:id;', [':id' => $playerid]);
                else {
                    $idxs = implode(',', $updateIdxs);
                    $cmd = $db->createCommand("update `PlayerKeys` set `GroupID`=null where  `PlayerID`=:id and `Index` not in ($idxs);", [':id' => $playerid]);
                }
                $cmd->execute();
            }catch (Exception $e){
                return $this->response(Error::STATUS_SUCCESS);
            }

            if($player->SessionType == 10 && isset($data['playtime']) && is_array($data['playtime'])) {
                $time = '00:00:00';
                $onetime = 0;
                $length = null;
                $scheduleid = 0;
                $insert = $db->createCommand('insert into PlaySessionTime(`SessionID`, `WeekDay`, `PlayTime`, `TempFlag`, `Length`)
                        values (' . $player->SessionID . ', :weekday, :time, :onetime, :length)
                        on duplicate key update `SessionID` = VALUES(`SessionID`);');
                $insert = $insert->bindParam(':weekday', $weekday);
                $insert = $insert->bindParam(':time', $time);
                $insert = $insert->bindParam(':onetime', $onetime);
                $insert = $insert->bindParam(':length', $length);

                $update = $db->createCommand('update PlaySessionTime set
                                        `SessionID` = ' . $player->SessionID . ',
                                        `WeekDay` = :weekday,
                                        `PlayTime` = :time,
                                        `TempFlag`=:onetime ,
                                        `Length`=:length
                                        where `ScheduleID`=:scheduleid;');
                $update = $update->bindParam(':weekday', $weekday);
                $update = $update->bindParam(':time', $time);
                $update = $update->bindParam(':onetime', $onetime);
                $update = $update->bindParam(':scheduleid', $scheduleid);
                $update = $update->bindParam(':length', $length);
                $firstInsertId = 4294967295;
                $updateId = [];
                foreach ($data['playtime'] as $value) {
                    if (!preg_match('/^[0-9][0-9,]*$/', $value['WeekDay']) ||
                        !preg_match('/^[0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}$/', $value['PlayTime'])
                    )
                        continue;
                    $time = '2012-03-03 ' . $value['PlayTime'];
                    $days = explode(',', $value['WeekDay']);
                    $tempflags = explode(',', $value['TempFlag']);
                    $length = $value['Length'];
                    if (count($days) != count($tempflags))
                        continue;
                    foreach ($days as $idx => $weekday) {
                        $onetime = $tempflags[$idx];
                        try {
                            $playtimeModel = PlaySessionTime::find()->select(['ScheduleID'])
                                ->where(['SessionID' => $player->SessionID, 'WeekDay' => $weekday])
                                ->andWhere("time(`PlayTime`) = time('$time')")
                                ->one();
                            $ret[] = $playtimeModel;
                            if (empty($playtimeModel)) {
                                $insert->execute();
                                $lastInsertId = Yii::$app->db->lastInsertID;
                                if ($lastInsertId < $firstInsertId)
                                    $firstInsertId = $lastInsertId;
                            } else {
                                $scheduleid = $playtimeModel->ScheduleID;
                                if (is_numeric($scheduleid)) {
                                    $update->execute();
                                    $updateId[] = $scheduleid;
                                }
                            }
                        }catch (Exception $e){}
                    }
                }
                if (!empty($updateId)) {
                    $updateId = implode(',', $updateId);
                    Yii::$app->db->createCommand("delete from PlaySessionTime where SessionID=:sessionid and ScheduleID not in ($updateId) and ScheduleID<:firstInsertId;", [
                        ':sessionid' => $player->SessionID,
                        ':firstInsertId' => $firstInsertId
                    ])->execute();
                    Yii::$app->db->createCommand("update PlayerStatus set SessionID= 0 where SessionID = :sessionid and ScheduleID not in ($updateId) and ScheduleID<:firstInsertId;;", [
                        ':sessionid' => $player->SessionID,
                        ':firstInsertId' => $firstInsertId
                    ])->execute();
                } else {
                    Yii::$app->db->createCommand("delete from PlaySessionTime where SessionID=:sessionid and ScheduleID<:firstInsertId;", [
                        ':sessionid' => $player->SessionID,
                        ':firstInsertId' => $firstInsertId
                    ])->execute();
                    Yii::$app->db->createCommand("update PlayerStatus set SessionID= 0 where SessionID = :sessionid and ScheduleID<:firstInsertId;;", [
                        ':sessionid' => $player->SessionID,
                        ':firstInsertId' => $firstInsertId
                    ])->execute();
                }
            }
            $detail = Yii::$app->db->createCommand("select a.PlayerID, a.SessionName, a.Priority, a.AudioGain, a.OwnerName, a.StreamType, a.SessionType, c.SessionID, c.ScheduleID, b.IpAddress, b.PlayType from v_session_info a
                                                    left join PlayerStatus b on a.PlayerID  = b.PlayerID
                                                    left join PlayerStatus c on c.SessionID  = a.SessionID
                                                    where a.PlayerID = $playerid;")
                ->queryOne();
            if(!empty($detail)){
                $detail['AudioGain'] = intval($detail['AudioGain']);
                $detail['PlayType'] = intval($detail['PlayType']);
                $detail['StreamType'] = intval($detail['StreamType']);
                $detail['SessionType'] = intval($detail['SessionType']);
                $detail['SessionID'] = intval($detail['SessionID']);
                return $this->response(Error::STATUS_SUCCESS, ['player' => $detail]);
            }
        }
        return $this->response(Error::STATUS_FAILED);
    }

    /**
     * delete realtime player
     * @param $id
     * @return array
     */
    public function actionRealtimeDelete($id) {
        $playingSession = VPlayingSessions::findOne(['SessionID' => $id]);
        if(!empty($playingSession))
            return $this->response(Error::STATUS_SESSION_ISPLAYING);
        $session = VSessionInfo::findOne(['PlayerID' => $id]);
        if(is_null($session))
            return $this->response(Error::STATUS_SESSION_NOT_EXISTS);
        $this->logTitle['title1'] = $session->PlayerID;
        $db = Yii::$app->db;
        try{
            if(isset($session->PlayerAddress))
                return $this->response(Error::STATUS_PLAYER_DELETE_ONLINE);
            if(isset($session->PlayerType) && $session->PlayerType == 2)
                return $this->response(Error::STATUS_PC_PLAYER_DELETE_FORBID);
            $db->createCommand()->delete('PlaySessionTime', ['SessionID' => $session->SessionID])
                ->execute();
            $db->createCommand()->delete('Players', ['PlayerID' => $session->PlayerID])
                ->execute();
            return $this->response(Error::STATUS_SUCCESS);
        }catch (Exception $e){}
        return $this->response(Error::STATUS_FAILED);
    }
    /**
     * get session detail
     */
    public function actionView($id){
        $query = VSessionInfo::find()->where(['SessionID' => $id]);
        if (($this->serverPri & 0x06) == 0)
            $query = $query->andWhere(['Type' => 'ctrl']);
        else if(($this->serverPri & 0x04) == 0)
            $query = $query->andWhere(['or', ['StreamType' => null], ['StreamType' => [0, 1, 128, 129]]]);
        $ret['session'] = $query->one();
        if(is_null($ret['session']))
            return $this->response(Error::STATUS_SESSION_NOT_EXISTS);

        //get session devices
        if(!is_null($ret['session']->BrdMode) && $ret['session']->MemberMode == 1){
            $ret['devices'] = Yii::$app->getDb()
                ->createCommand('select group_concat(DeviceID) as devices from BrdModeMember where BrdModeID=' . $ret['session']->BrdMode . ';')
                ->queryOne()['devices'];
        }
        else if((!empty($ret['session']->PlayerID)) and $ret['session']->SessionType == 1 and is_null($ret['session']->Owner)) {
            $ret['devices'] = Yii::$app->getDb()
                ->createCommand("select group_concat(device) as devices from (
                                        select a.DevID as device from DeviceList a, PlayerKeys b
                                         where b.GroupDisable = 0 and b.GroupID is not null and
                                         sp_dev_belong_group(a.DevID, b.GroupID) = true and
                                          b.PlayerID = '" . $ret['session']->PlayerID . "'
                                          group by a.DevID) c;")
                ->queryOne()['devices'];
        }
        else {
            $ret['devices'] = Yii::$app->getDb()
                ->createCommand('select group_concat(DeviceID) as devices from SessionDevice where SessionID=' . $id . ';')
                ->queryOne()['devices'];
        }
        //get session files
        $ret['files'] = VSessionFiles::find()->select(['FileID', 'FileName', 'TotalTime', 'FileFmt'])
                        ->andWhere(['SessionID' => $id])
                        ->all();
        //get play time
        $ret['playtime'] = Yii::$app->getDb()
                                ->createCommand("select CAST(`b`.`PlayTime` AS TIME) AS `PlayTime`, GROUP_CONCAT(`b`.`WeekDay` SEPARATOR ',') AS `WeekDay`,
                                                GROUP_CONCAT(`b`.`ScheduleID` SEPARATOR ',') AS `ScheduleID`,
                                               GROUP_CONCAT(`b`.`TempFlag` SEPARATOR ',') AS `TempFlag`
                                                FROM `PlaySessionTime` `b`
                                                LEFT JOIN `PlaySessions` `a` ON `b`.`SessionID` = `a`.`SessionID`
                                                WHERE `b`.`SessionID` = $id
                                                GROUP BY `a`.`SessionID` , `b`.`PlayTime`
                                                HAVING (COUNT(0) >= 1);")
                                ->queryAll();
        return $this->response(Error::STATUS_SUCCESS, $ret);
    }

    /**
     * delete session
     */
    public function actionDelete($id){
        $playingSession = VPlayingSessions::findOne(['SessionID' => $id]);
        if(!empty($playingSession))
            return $this->response(Error::STATUS_SESSION_ISPLAYING);

        $session = VSessionInfo::findOne(['SessionID' => $id]);
        if(is_null($session))
            return $this->response(Error::STATUS_SESSION_NOT_EXISTS);
        if (($session->SessionType == 2 || $session->SessionType == 9) && !$this->isSuper && !in_array('manual', $this->userinfo['pri'])) {
            return $this->response(Error::STATUS_PRIVILEGE_ERROR);
        }
        $this->logTitle['title1'] = $session->SessionName;
        $db = Yii::$app->db;
        try{

            if(isset($session->PlayerID)) {
                if(isset($session->PlayerAddress))
                    return $this->response(Error::STATUS_PLAYER_DELETE_ONLINE);
                if(isset($session->PlayerType) && $session->PlayerType == 2)
                    return $this->response(Error::STATUS_PC_PLAYER_DELETE_FORBID);
                $db->createCommand()->delete('PlaySessionTime', ['SessionID' => $id])
                    ->execute();
                $db->createCommand()->delete('Players', ['PlayerID' => $session->PlayerID])
                    ->execute();
            }
            else{
                $db->createCommand()->delete('PlaySessionTime', ['SessionID' => $id])
                    ->execute();
                $db->createCommand()->delete('PlaySessions', ['SessionID' => $id])
                    ->execute();
            }

            $db->createCommand("call sp_session_ctrl(:id, 0, 0);", [':id' => $id])
                ->execute();
            Yii::$app->db->createCommand("call sp_session_clean_up();")->execute();
            return $this->response(Error::STATUS_SUCCESS);
        }catch (Exception $e){}
        return $this->response(Error::STATUS_FAILED);
    }

    /**
     * create new playtime
     * @param $id SESSIONID
     * @return array
     */
    public function actionScheduleNew($id) {
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !isset($data['TempFlag']) || !isset($data['PlayTime']) || !isset($data['WeekDay']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $session = VSessionInfo::findOne(['SessionID' => $id]);
        if(is_null($session))
            return $this->response(Error::STATUS_SESSION_NOT_EXISTS);
        $this->logTitle['title1'] = $session->SessionName;
        $weekday = explode(',', $data['WeekDay']);
        if(empty($weekday))
            return $this->response(Error::STATUS_FAILED);
        foreach ($weekday as $day) {
            $this->logDetail['weekday' . $day] = $data['PlayTime'];
        }
        $tempflags = explode(',', $data['TempFlag']);
        if(empty($tempflags))
            return $this->response(Error::STATUS_FAILED);
        $success = [];
        $playtime = '2012-03-03 ' . $data['PlayTime'];
        $day = 3;
        $tempflag = 0;
        $cmd = Yii::$app->db->createCommand("INSERT INTO PlaySessionTime (SessionID, WeekDay,PlayTime,TempFlag)
                                  values (:session, :weekday, :playtime, :onetime)
                                 on duplicate key update TempFlag = values(TempFlag);");
        $cmd = $cmd->bindParam(':session', $id);
        $cmd = $cmd->bindParam(':weekday', $day);
        $cmd = $cmd->bindParam(':playtime', $playtime);
        $cmd = $cmd->bindParam(':onetime', $tempflag);
        foreach ($weekday as $idx => $day) {
            if(is_numeric($day))
                $success[] = $day;
            if(isset($tempflags[$idx]) && is_numeric($tempflags[$idx]))
                $tempflag = $tempflags[$idx];
            else
                $tempflag = 0;
            $cmd->execute();
        }
        $sessionTimes = VTimedSessionInfo::find()->select(['ScheduleID', 'PlayTime', 'WeekDay', 'TempFlag'])
            ->where(['SessionID' => $session->SessionID, 'PlayTime' => $data['PlayTime']])
            ->one();
        return $this->response(Error::STATUS_SUCCESS, $sessionTimes);
    }

    /**
     * delete schedule id from PlaySessionTime
     * @param $id SESSIONID
     * @param $playtime playtime hh:mm:ss
     * @param $weekday weekday 0-6
     * @return array
     */
    public function actionScheduleDelete($id, $scheduleid) {
        $playingSession = VPlayingSessions::findOne(['SessionID' => $id]);
        if(!empty($playingSession))
            return $this->response(Error::STATUS_SESSION_ISPLAYING);
        try{
            $session = PlaySessions::findOne(['SessionID' => $id]);
            $schedule = PlaySessionTime::findOne(['ScheduleID' => $scheduleid]);
            if(empty($session) || empty($schedule)) {
                $this->logSended = true;
                return $this->response(Error::STATUS_SUCCESS);
            }
            $this->logTitle['title1'] = $session->SessionName;
            $this->logDetail['weekday' . $schedule->WeekDay] = (preg_split('/ /', $schedule->PlayTime))[1];
            Yii::$app->db->createCommand('call delete_task(:scheduleid);', [':scheduleid' => $scheduleid])
                ->execute();
            Yii::$app->db->createCommand("update PlayerStatus set SessionID= 0 where SessionID = :sessionid and ScheduleID  = :scheduleid;", [
                ':sessionid' => $id,
                ':scheduleid' => $scheduleid
            ])->execute();
            Yii::$app->db->createCommand("call sp_session_clean_up();")->execute();
            return $this->response(Error::STATUS_SUCCESS);
        }catch (Exception $e){}
        return $this->response(Error::STATUS_FAILED);
    }

    /**
     * @param $id
     * @param $scheduleid
     * @return array
     */
    public function actionScheduleUpdate($id, $scheduleid){
        $data = Yii::$app->request->post('data');
        if(is_null($data) || !isset($data['WeekDay']) || empty($data['PlayTime']) || !isset($data['TempFlag']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        if(!preg_match('/^[0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}$/', $data['PlayTime']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        else {
            $playtime = $data['PlayTime'];
            $data['PlayTime'] = '2012-03-03 ' . $data['PlayTime'];
        }
        unset($data['ScheduleID']);
        $session = PlaySessions::findOne(['SessionID' => $id]);
        if(is_null($session))
            return $this->response(Error::STATUS_SESSION_NOT_EXISTS);
        $schedule = PlaySessionTime::findOne(['ScheduleID' => $scheduleid]);
        if(is_null($schedule) || $schedule->SessionID != $id)
            return $this->response(Error::STATUS_SCHEDULE_NOT_EXISTS);
        $this->logTitle['title1'] = $session->SessionName;
        $this->logDetail['weekday' . $data['WeekDay']] = $playtime;
        if($schedule->load(['data' => $data], 'data') && $schedule->save()) {
            Yii::$app->db->createCommand("call sp_session_clean_up();")->execute();
            return $this->response(Error::STATUS_SUCCESS, $schedule->toArray(['ScheduleID', 'PlayTime', 'WeekDay', 'TempFlag']));
        }
        return $this->response(Error::STATUS_FAILED);
    }

    public function actionPlaytimeDelete($id, $playtime){
        $playingSession = VPlayingSessions::findOne(['SessionID' => $id]);
        if(!empty($playingSession))
            return $this->response(Error::STATUS_SESSION_ISPLAYING);
        if(!preg_match('/^[0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}$/', $playtime))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $session = VSessionInfo::findOne(['SessionID' => $id]);
        if(is_null($session))
            return $this->response(Error::STATUS_SESSION_NOT_EXISTS);
        $this->logTitle['title1'] = $session->SessionName;
        $this->logTitle['title2'] = $playtime;
        $models = PlaySessionTime::find()->select(['ScheduleID'])
            ->where(['SessionID' => $id])
            ->andWhere("time(`PlayTime`)=time('$playtime')")
            ->all();
        $modelIds = [];
        foreach ($models as $model)
            $modelIds[] = $model->ScheduleID;
        if(count($modelIds) > 0) {
            $scheduleid = implode(',', $modelIds);
            Yii::$app->db
                ->createCommand("delete from `PlaySessionTime` where `SessionID`= $id and time(`PlayTime`)=time(:playtime);", [':playtime' => $playtime])
                ->execute();
            Yii::$app->db->createCommand("update PlayerStatus set SessionID= 0 where SessionID = :sessionid and ScheduleID in ($scheduleid);", [
                ':sessionid' => $id
            ])->execute();
            Yii::$app->db->createCommand("call sp_session_clean_up();")->execute();
        }
        return $this->response(Error::STATUS_SUCCESS);
    }

    public function actionPlaytimeUpdate($id, $old_playtime, $new_playtime){
        if(!preg_match('/^[0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}$/', $old_playtime) ||
            !preg_match('/^[0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}$/', $new_playtime))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $session = VSessionInfo::findOne(['SessionID' => $id]);
        if(is_null($session))
            return $this->response(Error::STATUS_SESSION_NOT_EXISTS);
        $playtimeCnt = PlaySessionTime::find()->where(['SessionID' => $id])
            ->andWhere("time(`PlayTime`)='$old_playtime'")
            ->count();
        if($playtimeCnt == 0) {
            $this->logSended = true;
            return $this->response(Error::STATUS_FAILED);
        }
        try{
            Yii::$app->db
                ->createCommand("update `PlaySessionTime` set `PlayTime` = :newPlaytime where `SessionID`= $id and time(`PlayTime`)=:oldPlaytime;",
                    [':oldPlaytime' => $old_playtime, ':newPlaytime' => '2012-03-03 ' . $new_playtime])
                ->execute();
            $this->logTitle['title1'] = $session->SessionName;
            $this->logDetail['ele/oldtime'] = $old_playtime;
            $this->logDetail['ele/newtime'] = $new_playtime;
            Yii::$app->db->createCommand("call sp_session_clean_up();")->execute();
            return $this->response(Error::STATUS_SUCCESS);
        }catch (\Exception $e){}
        $this->logSended = true;
        return $this->response(Error::STATUS_FAILED);
    }

    /**
     * create or update session
     *
     * @param [VSessionInfo] $vsession
     * @param [post data] $data
     * @return real session(PlaySessions)
     */
    private function sessionNP($vsession, $data){
        // pri check
        if ((!empty($data['RfDevCtrl']) || !empty($data['SysEnabled'])) &&
            (!in_array('audioBroadcast', Yii::$app->getCache()->get('srvPri')) ||
            !in_array('videoBroadcast', Yii::$app->getCache()->get('srvPri')))) {
            return $this->response(Error::STATUS_PRIVILEGE_ERROR);
        }
        if (isset($vsession->RfDevCtrl))
            $vsession->RfDevCtrl = hex2bin($vsession->RfDevCtrl);
        else
            $vsession->RfDevCtrl = null;
        if(is_null($vsession->StreamType) || ($vsession->StreamType & 0x7c) != 0 || is_null($vsession->SessionType) || !in_array($vsession->SessionType, [0,1,2,8,9,10]))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $db = Yii::$app->getDb();
        if($vsession->isNewRecord) {
            $real = new PlaySessions();
            $vsession->SessionID = $db->createCommand('select get_idle_sessionid(true) as id;')
                                ->queryOne()['id'];
            $playerID = null;
            $streamType = 0;
        }
        else {
            $real = PlaySessions::findOne(['SessionID' => $vsession->SessionID]);
            if (is_null($real))
                return $this->response(Error::STATUS_SESSION_NOT_EXISTS);
            $playerID = $real->PlayerID;
            $streamType = $real->StreamType;
        }
        $id = $vsession->SessionID;
        $vsession = $vsession->toArray();
        foreach ($real as $key => $value) {
            if (array_key_exists($key, $vsession))
                $real->$key = $vsession[$key];
        }
        $isNew = $real->isNewRecord;
        if($real->isNewRecord) {
            $real->Owner = $this->isAdmin ? null: Yii::$app->user->getId();
            if (($real->SessionType == 0 || $real->SessionType == 8)) {
                if(Yii::$app->base->activeTemplateId == -1)
                    return $this->response(Error::STATUS_ACTIVE_TEMPLATE_NOT_SET);
            }
        }

        //player id can't set by user
        $real->PlayerID = $playerID;
        if(!empty($playerID))
            $real->StreamType = ($real->StreamType & 0x80) + ($streamType & 0x7f);
        //StreamType and SessionType must be zero from control session
        if (($real->StreamType & 0x7f) !== 0 || ($real->SessionType !== 0 && $real->SessionType != 2)) {
            $real->SysEnabled = null;
            $real->RfDevCtrl = null;
        }

        if (is_null($real->BrdMode))
            $real->MemberMode = 0;
        else {
            $templateNow = Yii::$app->base->activeTemplateId;
            if($templateNow == -1)
                return $this->response(Error::STATUS_ACTIVE_TEMPLATE_NOT_SET);
            $real->Template = $templateNow;
        }

        if($real->StreamType == 3 && ($this->serverPri & 0x04) != 0x04)
            return $this->response(Error::STATUS_FAILED);
        if($real->StreamType == 1 && ($this->serverPri & 0x02) != 0x02)
            return $this->response(Error::STATUS_FAILED);
        if (!$real->save())
            return $this->response(Error::STATUS_FAILED);
        //session device
        $sql = 'delete from SessionDevice where SessionID = ' . $id . ' ';
        if ((!is_null($real->BrdMode) && $real->MemberMode == 1) || empty($data['devices']))
            $db->createCommand($sql . ';')->execute();
        else {
            $db->createCommand($sql . 'and DeviceID not in (' . $data['devices'] . ');')->execute();
            $db->createCommand('insert into `SessionDevice` (`SessionID`, `DeviceID`)
                                select ' . $id . ', `DevID` from `DeviceList` where `DevID` in (' . $data['devices'] . ')
                                on duplicate key update `SessionID` = values(`SessionID`);')
                ->execute();
        }

        //session files
        if(($real->StreamType & 0x7f) != 0 && in_array($real->SessionType, [0,2,8,9])) {
            $db->createCommand('delete from PlayListFiles where SessionID = ' . $id . ';')
                ->execute();
            if (($real->StreamType & 0x7f) !== 0 && isset($data['files']) && isset($data['files']) && is_array($data['files'])) {
                $query = $db->createCommand('insert into PlayListFiles(`FileID`, `SequenceID`, `SessionID`)
                                values (:fileid, :sequence, ' . $id . ')
                                on duplicate key update `SessionID` = VALUES(`SessionID`);');
                $query = $query->bindParam(':fileid', $fileid);
                $query = $query->bindParam(':sequence', $row);
                $row = 0;
                foreach ($data['files'] as $file) {
                    $fileid = $file['FileID'];
                    $query->execute();
                    $row++;
                }
            }
        }

        //play time
        if (($real->SessionType == 0 || $real->SessionType == 8)) {
            if(isset($data['playtime']) && is_array($data['playtime'])) {
                $time = '00:00:00';
                $onetime = 0;
                $scheduleid = 0;
                $insert = $db->createCommand('insert into PlaySessionTime(`SessionID`, `WeekDay`, `PlayTime`, `TempFlag`)
                                values (' . $id . ', :weekday, :time, :onetime)
                                on duplicate key update `SessionID` = VALUES(`SessionID`);');
                $insert = $insert->bindParam(':weekday', $weekday);
                $insert = $insert->bindParam(':time', $time);
                $insert = $insert->bindParam(':onetime', $onetime);

                $update = $db->createCommand('update PlaySessionTime set
                                                `SessionID` = ' . $id . ',
                                                `WeekDay` = :weekday,
                                                `PlayTime` = :time,
                                                `TempFlag`=:onetime 
                                                where `ScheduleID`=:scheduleid;');
                $update = $update->bindParam(':weekday', $weekday);
                $update = $update->bindParam(':time', $time);
                $update = $update->bindParam(':onetime', $onetime);
                $update = $update->bindParam(':scheduleid', $scheduleid);
                $firstInsertId = 4294967295;
                $updateId = [];
                foreach ($data['playtime'] as $value) {
                    if (!preg_match('/^[0-9][0-9,]*$/', $value['WeekDay']) ||
                        !preg_match('/^[0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}$/', $value['PlayTime'])
                    )
                        continue;
                    $time = '2012-03-03 ' . $value['PlayTime'];
                    $days = explode(',', $value['WeekDay']);
                    $tempflags = explode(',', $value['TempFlag']);
                    if (count($days) != count($tempflags))
                        continue;
                    foreach ($days as $idx => $weekday) {
                        $onetime = $tempflags[$idx];
                        try {
                            $playtimeModel = PlaySessionTime::find()->select(['ScheduleID'])
                                ->where(['SessionID' => $id, 'WeekDay' => $weekday])
                                ->andWhere("time(`PlayTime`) = time('$time')")
                                ->one();
                            $ret[] = $playtimeModel;
                            if (empty($playtimeModel)) {
                                $insert->execute();
                                $lastInsertId = Yii::$app->db->lastInsertID;
                                if ($lastInsertId < $firstInsertId)
                                    $firstInsertId = $lastInsertId;
                            } else {
                                $scheduleid = $playtimeModel->ScheduleID;
                                if (is_numeric($scheduleid)) {
                                    $update->execute();
                                    $updateId[] = $scheduleid;
                                }
                            }
                        }catch (Exception $e){}
                    }
                }
                if(!$isNew) {
                    if (!empty($updateId)) {
                        $updateId = implode(',', $updateId);
                        Yii::$app->db->createCommand("delete from PlaySessionTime where SessionID=:sessionid and ScheduleID not in ($updateId) and ScheduleID<:firstInsertId;", [
                            ':sessionid' => $id,
                            ':firstInsertId' => $firstInsertId
                        ])->execute();
                        Yii::$app->db->createCommand("update PlayerStatus set SessionID= 0 where SessionID = :sessionid and ScheduleID not in ($updateId) and ScheduleID<:firstInsertId;;", [
                            ':sessionid' => $id,
                            ':firstInsertId' => $firstInsertId
                        ])->execute();
                    } else {
                        Yii::$app->db->createCommand("delete from PlaySessionTime where SessionID=:sessionid and ScheduleID<:firstInsertId;", [
                            ':sessionid' => $id,
                            ':firstInsertId' => $firstInsertId
                        ])->execute();
                        Yii::$app->db->createCommand("update PlayerStatus set SessionID= 0 where SessionID = :sessionid and ScheduleID<:firstInsertId;;", [
                            ':sessionid' => $id,
                            ':firstInsertId' => $firstInsertId
                        ])->execute();
                    }
                }
            }
        }
        else
            PlaySessionTime::deleteAll(['SessionID' => $id]);

        $lists = VTimedSessionInfo::find()->where(['SessionID' => $id])->all();
        if(empty($lists))
            $lists = VSessionInfo::find()->where(['SessionID' => $id])->all();
        if(count($lists) > 0) {
            $status = VPlayingSessions::find()->select(['SessionID', 'FileName', 'CurrTotalProgress', 'Weekday', 'PlayTime'])
                ->where(['in', 'SessionID', [$id]])
                ->all();
        }
        else
            $status = [];
        Yii::$app->db->createCommand("call sp_session_clean_up();")->execute();
        return $this->response(Error::STATUS_SUCCESS, ['count' => count($lists), 'list' => $lists, 'status' => $status]);
    }
    /**
     * update session
     */
    public function actionUpdate($id){
        $data = Yii::$app->getRequest()->post('data');
        if (is_null($data) || is_null($data['session']) ||
            (isset($data['session']['RfDevCtrl']) && (strlen($data['session']['RfDevCtrl']) != 32 || !ctype_xdigit($data['session']['RfDevCtrl']))) ||
            (!empty($data['devices']) && !preg_match('/^[0-9][0-9,]+[0-9]$/', $data['devices']))
            )
            return $this->response(Error::STATUS_PARAM_ERROR);
        $session = VSessionInfo::findOne(['SessionID' => $id]);
        if(is_null($session))
            return $this->response(Error::STATUS_SESSION_NOT_EXISTS);
        if (($session->SessionType == 2 || $session->SessionType == 9) && !$this->isSuper && !in_array('manual', $this->userinfo['pri'])) {
            return $this->response(Error::STATUS_PRIVILEGE_ERROR);
        }
        unset($data['session']['BrdMode'], $data['session']['Template'], $data['session']['Owner']);
        if(!$session->load(Yii::$app->getRequest()->post('data'), 'session'))
            return $this->response(Error::STATUS_FAILED);
        $this->logTitle['title1'] = $session->SessionName;
        return $this->sessionNP($session, $data);
    }

    /**
     * create new session
     */
    public function actionNew(){
        $data = Yii::$app->getRequest()->post('data');
        if (is_null($data) || is_null($data['session']) ||
            (!empty($data['session']['RfDevCtrl']) && (strlen($data['session']['RfDevCtrl']) != 32 || !ctype_xdigit($data['session']['RfDevCtrl']))) ||
            (!empty($data['devices']) && !preg_match('/^[0-9][0-9,]+[0-9]$/', $data['devices'])))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $session = new VSessionInfo();
        if (!$session->load(Yii::$app->getRequest()->post('data'), 'session'))
            return $this->response(Error::STATUS_FAILED);
        if (($session->SessionType == 2 || $session->SessionType == 9) && !$this->isSuper && !in_array('manual', $this->userinfo['pri'])) {
            return $this->response(Error::STATUS_PRIVILEGE_ERROR);
        }
        $this->logTitle['title1'] = $session->SessionName;
        return $this->sessionNP($session, $data);
    }

    /**
     * @param $id session id
     * @param $state control state: can be 'start'/'stop'/'pause'
     * @param $schedule schedule id,for manual task ,set zero
     */
    public function actionCtrl($id , $state, $schedule) {
        $session = PlaySessions::findOne(['SessionID' => $id]);
        if(is_null($session)) {
            $this->logSended = true;
            return $this->response(Error::STATUS_SESSION_NOT_EXISTS);
        }
        $this->logTitle['title1'] = $session->SessionName;
        $this->logTitle['title2'] = $state;
        if (($session->SessionType == 2 || $session->SessionType == 9) && !$this->isSuper && !in_array('manual', $this->userinfo['pri'])) {
            return $this->response(Error::STATUS_PRIVILEGE_ERROR);
        }
        if(empty($session->PlayerID)) {
            if(!$this->isAdmin && !$this->isSuper && Yii::$app->user->getId() != $session->Owner)
                return $this->response(Error::STATUS_PRIVILEGE_ERROR);
        }

        if($state == 'start') {
            Yii::$app->db->createCommand("select sp_session_start(SessionID, :schedule , 0, StreamType, BrdMode, MemberMode, SysEnabled,  RfDevCtrl)" .
                " from  PlaySessions where  SessionID =:id;", [':schedule' => $schedule, ':id' => $id])
                ->execute();
        }
        else {
            if($state == 'pause')
                $ctrl = 2;
            else //pause
                $ctrl = 0;
            Yii::$app->db->createCommand("call sp_session_ctrl(:id, :ctrl, :schedule);", [':id' => $id, ':ctrl' => $ctrl, 'schedule' => null])
                ->execute();
        }
        return $this->response(Error::STATUS_SUCCESS);
    }
    /**
     * return template and model list
     */
    public function actionMenuList(){
        $templates = TaskTemplate::find()->all();
        $enableTemplate = Yii::$app->base->activeTemplateId;
        $modes = BroadcastMode::find()->all();
        return $this->response(Error::STATUS_SUCCESS, [
            'templates' => [
                'list' => $templates,
                'active' => $enableTemplate,
            ],
            'modes' => $modes,
            'weekday' => Yii::$app->base->weekday
        ]);
    }

    /**
     *
     * @return array
     */
    public function actionStatus(){
        $status = VPlayingSessions::find()
            ->select(['SessionID', 'FileName', 'CurrTotalProgress', 'Pause', 'PlayTime', 'WeekDay', 'Template', 'BrdMode', 'SessionType', 'StreamType'])
            ->indexBy('SessionID')
            ->orderBy(['PlayTime' => SORT_ASC])
            ->all();
        foreach ($status as $st){
            $arr = $st->toArray();
            $arr['status'] = 'change';
            $ret[] = $arr;
        }
        return $this->response(Error::STATUS_SESSION_STATUS_CHANGED, ['action' => 'flush', 'status' => $ret]);
    }

    private function getPlayerIp($id, &$ip) {
        $sql = "SELECT PlayerID, IpAddress FROM PlayerStatus WHERE PlayerID = '$id'";
        $player = Yii::$app->db->createCommand($sql)->queryOne();
        if (empty($player) || empty($player['IpAddress'])) {
            $sql = "SELECT DeviceID as PlayerID, IpAddress FROM DeviceStatus WHERE DeviceID = '$id'";
            $player = Yii::$app->db->createCommand($sql)->queryOne();
            if(empty($player) || empty($player['IpAddress']))
                return Error::STATUS_PLAYER_OFFLINE;
        }
        $ip = $player['IpAddress'];
        return Error::STATUS_SUCCESS;
    }

    public function actionBw80xPlayCtrl () {
        $data = Yii::$app->request->post('data');
        if (empty($data) || empty($data['id']) || !preg_match('/^[0-9][0-9]*[0-9]$/', $data['id'])) {
            return $this->response(Error::STATUS_PARAM_ERROR);
        }

        $session = PlaySessions::find()->select(['SessionName'])->where(['PlayerID' => $data['id']])
            ->asArray()->one();
        if(is_null($session)) {
            $session = DeviceList::find()->select(['SessionName'=>'Dev_Name'])->where(['DevID' => $data['id']])
                ->asArray()->one();
            if(is_null($session)) {
                $this->logSended = true;
                return $this->response(Error::STATUS_SESSION_NOT_EXISTS);
            }
        }
        $this->logTitle['title1'] = $session['SessionName'];

        $ip = null;
        $ret = $this->getPlayerIp($data['id'], $ip);
        if($ret)
            return $this->response($ret);
        $ret = $this->playerCtrl($ip, $data);
        if (!empty($ret))
            return $this->response(Error::STATUS_SUCCESS, $ret);
        return $this->response(Error::STATUS_FAILED, ['data' => $ret]);
    }

    private function bw801GrpCtrl ($ip, $grp) {
        $ParamArr = ['brdGrpCtrl='.$grp];
        $url = "http://webApp:webApp@" . $ip . "/cgi-bin/playerbrdgrpset.cgi";
        $ch = curl_init ();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, implode("&", $ParamArr));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 15);
        curl_setopt($ch, CURLOPT_HEADER, false);
        $res = curl_exec ( $ch );
        return $res;
    }
    /**
     * @param $ip
     * @param $brdCtrl 0：关闭，1：打开（不传为不修改） (打开广播)
     * @param $line1Input 0：关闭，1：打开（不传为不修改） (输入通道)
     * @param $line2Input 0：关闭，1：打开（不传为不修改） (输入通道)
     * @param $micInput 0：关闭，1：打开（不传为不修改） (输入通道)
     * @param $mp3PlayCtrl 0：停止，1：播放（不传为不修改）, 2：暂停 （不传为不修改） （播放MP3文件）
     * @param $mp3FileChange 0:当前文件，1：上一首，2：下一首，3：指定MP3文件（不传为不修改）
     * @param $dirId  0（指定MP3文件所在的目录号 - mp3FileChange 指定为3时必填）
     * @param $fileId  0（指定MP3文件所在目录的文件ID号 - mp3FileChange 指定为3时必填）
     */
    public function playerCtrl ($ip, $data) {
        $ParamArr = array ();
        $state = '';
        if (array_key_exists( "brdGrpCtrl", $data)) {
            $this->logTitle['title1'] .= '_group';
            $open = 0;
            $group = $data['brdGrpCtrl'] + 0;
            while($group){
                $open++;
                $group = $group & ($group-1);
            }
            $this->logTitle['title2'] = 'open:' . $open;
            $res = $this->bw801GrpCtrl($ip, $data['brdGrpCtrl']);
            if(!empty($res))
                return $res;
        }
        if (array_key_exists( "brdCtrl", $data) && in_array ( $data['brdCtrl'], array ("0", "1" ) )) {
            $ParamArr [] = "brdCtrl=" . $data['brdCtrl'];
            $this->logTitle['title1'] .= '_brd';
            $state = $data['brdCtrl'] == 1 ? 'start' : 'stop';
        }
        if (array_key_exists( "line1Input", $data) && in_array ( $data['line1Input'], array ("0", "1" ) )) {
            $ParamArr [] = "line1Input=" . $data['line1Input'];
            $this->logTitle['title1'] .= '_line1';
            $state = $data['line1Input'] == 1 ? 'start' : 'stop';
        }
        if (array_key_exists( "line2Input", $data) && in_array ( $data['line2Input'], array ("0", "1" ) )) {
            $ParamArr [] = "line2Input=" . $data['line2Input'];
            $this->logTitle['title1'] .= '_line2';
            $state = $data['line2Input'] == 1 ? 'start' : 'stop';
        }
        if (array_key_exists( "micInput", $data) && in_array ( $data['micInput'], array ("0", "1" ) )) {
            $ParamArr [] = "micInput=" . $data['micInput'];
            $this->logTitle['title1'] .= '_mic';
            $state = $data['micInput'] == 1 ? 'start' : 'stop';
        }
        if (array_key_exists( "mp3PlayCtrl", $data) && in_array ( $data['mp3PlayCtrl'], array ("0", "1", "2" ) )) {
            $ParamArr [] = "mp3PlayCtrl=" . $data['mp3PlayCtrl'];
            $this->logTitle['title1'] .= '_mp3';
            $state = $data['mp3PlayCtrl'] == 1 ? 'start' : 'stop';
        }
        if (array_key_exists( "mp3FileChange", $data) && in_array ( $data['mp3FileChange'], array ("0", "1", "2", "3" ) )) {

            if ($data['mp3FileChange'] == "3") {
                if (! empty ($data['dirId']) && ! empty ($data['fileId'])) {
                    $ParamArr [] = "mp3FileChange=" . $data['mp3FileChange'];
                    $ParamArr [] = "dirId=" . $data['dirId'];
                    $ParamArr [] = "fileId=" . $data['fileId'];
                }
            } else {
                $ParamArr [] = "mp3FileChange=" . $data['mp3FileChange'];
            }
            $this->logTitle['title1'] .= '_mp3FileChange';
        }
        if(empty($ParamArr))
            return true;
        $url = "http://webApp:webApp@" . $ip . "/cgi-bin/playerctrlset.cgi";
        $ch = curl_init ();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, implode("&", $ParamArr));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 15);
        curl_setopt($ch, CURLOPT_HEADER, false);
        $res = curl_exec ( $ch );
        curl_close ( $ch );

        $this->logTitle['title2'] = $state;
        return $res;
//        $res = str_replace(["\n", ",}", ',]'], ['', '}', ']'], $res);
//
//        return json_decode($res);
    }

    public function actionPlayerStatus() {
        $data = Yii::$app->request->post('data');
        if((!empty($data) && !empty($data['playerid']) && !preg_match('/^[0-9][0-9,]*[0-9]$/', $data['playerid'])))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $ret = [];
        if(!empty($data) && !empty($data['playerid'])) {
            $sql1 = "SELECT PlayerID, IpAddress from PlayerStatus where PlayerID in (" . $data['playerid'] .");";
            $sql2 = "SELECT DeviceID as PlayerID, IpAddress from DeviceStatus where DeviceID in (" . $data['playerid'] .") and DeviceID like '49%';";
            $ids = explode(',', $data['playerid']);
            foreach ($ids as $id) {
                $ret[$id] = null;
            }
        }
        else {
            $sql1 = "SELECT PlayerID, IpAddress from PlayerStatus where PlayType <> 8 and PlayType <> 0;";
            $sql2 = "SELECT DeviceID as PlayerID, IpAddress from DeviceStatus where DeviceID like '49%';";
        }
        $player1 = Yii::$app->db->createCommand($sql1)->queryAll();
        $player2 = Yii::$app->db->createCommand($sql2)->queryAll();
        if (!empty($player1) || !empty($player2)) {
            $players = array_merge($player1, $player2);
            foreach ($players as $p) {
                if (empty($p['IpAddress']))
                    continue;
                $url = "http://webApp:webApp@" . $p['IpAddress'] . "/cgi-bin/getplayerst.cgi";
                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $url);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 15);
                curl_setopt($ch, CURLOPT_HEADER, false);
                $playerSt = curl_exec($ch);
                $playerSt = str_replace(["\n", ",}", ',]'], ['', '}', ']'], $playerSt);
                curl_close($ch);
                $playerStatus = json_decode($playerSt, true);
                if (!empty($playerStatus) && is_array($playerStatus)) {
                    $ret[$p['PlayerID']] = $playerStatus[0];
                }
            }
        }
        return $this->response(Error::STATUS_SUCCESS, $ret);
    }


    public function actionDeviceUpdate () {
        $data = Yii::$app->getRequest()->post('data');
        if (empty($data) || !isset($data['sessionId']) || !array_key_exists('devices', $data) ||
            (!empty($data['devices']) && !preg_match('/^[0-9][0-9,]+[0-9]$/', $data['devices'])))
            return $this->response(Error::STATUS_PARAM_ERROR);
        if (isset($data['type']) && in_array($data['type'], ['add', 'remove']))
            $type = $data['type'];
        else
            $type = 'flush';
        $id = $data['sessionId'];
        $devices = $data['devices'];
        $db = Yii::$app->getDb();
        if ($type == 'flush') {
            //session device
            $sql = 'delete from SessionDevice where SessionID = ' . $id . ' ';
            if (empty($devices))
                $db->createCommand($sql . ';')->execute();
            else {
                $db->createCommand($sql . 'and DeviceID not in (' . $devices . ');')->execute();
                $db->createCommand('insert into `SessionDevice` (`SessionID`, `DeviceID`)
                                select ' . $id . ', `DevID` from `DeviceList` where `DevID` in (' . $devices . ')
                                on duplicate key update `SessionID` = values(`SessionID`);')
                    ->execute();
            }
        } else if ($type == 'add') {
            $db->createCommand('insert into `SessionDevice` (`SessionID`, `DeviceID`)
                                select ' . $id . ', `DevID` from `DeviceList` where `DevID` in (' . $devices . ')
                                on duplicate key update `SessionID` = values(`SessionID`);')
                ->execute();
        } else if ($type == 'remove') {
            $db->createCommand("delete from SessionDevice where SessionID = $id and DeviceID in ('$devices');")
                ->execute();
        } else {
            return $this->response(Error::STATUS_PARAM_ERROR);
        }

        $session = PlaySessions::find()->select(['SessionName'])->where(['SessionID' => $id])->one();
        if(is_null($session)) {
            $this->logSended = true;
            return $this->response(Error::STATUS_SESSION_NOT_EXISTS);
        }
        $this->logTitle['title1'] = $session->SessionName;
        $this->logDetail['ele/update'] = $devices;
        return $this->response(Error::STATUS_SUCCESS);
    }

    public function actionGetConfig(){
        $ret = ['audio_brd_delay'=>null];

        $info = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_AUDIO_DELAY]);
        if(!empty($info))
            $ret['audio_brd_delay'] = $info->InfoValue;

        return $this->response(Error::STATUS_SUCCESS, $ret);
    }

    public function actionSetConfig(){
        if(!$this->isSuper) {
            $this->logSended = true;
            return $this->response(Error::STATUS_PRIVILEGE_ERROR);
        }

        $data = Yii::$app->request->post('data');
        if(empty($data) || !is_numeric($data['audio_brd_delay']) || ($data['audio_brd_delay'] < 0))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $delay = floor($data['audio_brd_delay']);
        if($delay > 10)
            $delay = 10;
        Yii::$app->base->baseSet(Common::BASEINFO_AUDIO_DELAY, null, $delay);
        $this->logTitle['title1'] = $delay;

        return $this->actionGetConfig();
    }
}
