<?php
namespace api\controllers;

use common\classes\Common;
use common\classes\HttpRequest;
use common\models\BaseInfo;
use common\models\System;
use common\models\Users;
use Yii;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;
use common\models\LoginForm;
use common\classes\Error;
use common\models\Cache;
use common\models\UserDevices;
use yii\helpers\ArrayHelper;
use common\models\Users as User;
use yii\web\ForbiddenHttpException;
use yii\web\UploadedFile;

/**
 * User controller
 */
class UserController extends BaseController
{
    public $postRequest = ['list', 'update', 'new', 'delete', 'auth', 'update-mgr', 'unionid-bind', 'unionid-unbind'];

    /**
     * get detail infomation for current login user
     */
    public function actionViewMgr(){
        $user = Yii::$app->getUser();
        $user = User::findIdentity($user->getId());
        if(is_null($user))
            return $this->response(Error::STATUS_FAILED);
        // $user = User::findIdentity(0);
        $ret = $user->toArray(['User_Name', 'User_Desc', 'IcCard', 'Phone', 'User_Sn', 'UnionId']);
        if (!is_null($ret['UnionId']))
            $ret['UnionId'] = 'bind';
        else
            $ret['UnionId'] = 'unbind';
        return $this->response(Error::STATUS_SUCCESS, $ret);
    }

    public function actionUpdateMgr(){
        if (!array_key_exists('User_Desc', $this->data) || !array_key_exists('IcCard', $this->data)
            || !array_key_exists('Phone', $this->data) || !array_key_exists('User_Sn', $this->data)
            || !isset($this->data['Pass_Word']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $user = Yii::$app->getUser();
        $user = User::findIdentity($user->getId());
        if (is_null($user))
            return $this->response(Error::STATUS_FAILED);
        if ($user->Pass_Word != User::mysql_password($this->data['Pass_Word']))
            return $this->response(Error::STATUS_USER_PASSWORD_ERROR);
        if (isset($this->data['New_Pass_Word']))
            $user->Pass_Word = User::mysql_password($this->data['New_Pass_Word']);
        if(!empty($this->data['Phone'])){
            $count = Users::find()->where(['not', ['User_ID'=> $user->User_ID]])
                ->andWhere(['Phone'=>$this->data['Phone']])->count();
            if($count>0)
                return $this->response(Error::STATUS_USER_PHONE_EXIST);
        }
        $user->User_Desc = isset($this->data['User_Desc']) ? $this->data['User_Desc'] : null;
        $user->IcCard = (!empty($this->data['IcCard']))? trim($this->data['IcCard']) : null;
        $user->Phone = (!empty($this->data['Phone']))? trim($this->data['Phone']) : null;
        $user->User_Sn = (!empty($this->data['User_Sn']))? trim($this->data['User_Sn']) : null;
        if(!empty($user->IcCard)) {
            $count = User::find()->where(['not', ['User_Name' => $user->User_Name]])
                ->andWhere(['IcCard' => $user->IcCard])->count();
            if($count > 0)
                return $this->response(Error::STATUS_USER_ICCARD_EXIST);
        }
        if (!empty($user->User_Sn)) {
            $count = User::find()->where(['not', ['User_Name' => $user->User_Name]])
                ->andWhere(['User_Sn' => $user->User_Sn])->count();
            if ($count > 0)
                return $this->response(Error::STATUS_USER_USERSN_EXIST);
        }
        if($user->save())
            return $this->response(Error::STATUS_SUCCESS);
        else
            return $this->response(Error::STATUS_FAILED);
    }

    /**
     * bind new unionid
     */
    public function actionUnionidBind()
    {
        return $this->response(Error::STATUS_API_DEPRECATED);
    }

    /**
     * unbind unionid
     */
    public function actionUnionidUnbind()
    {
        return $this->response(Error::STATUS_API_DEPRECATED);
    }

    public function actionWxUnbind () {
        return $this->response(Error::STATUS_API_DEPRECATED);
    }

    public function actionDpmList() {
        $dpms = User::find()->select(['Dpm_Name'])
            ->where(['not', ['Dpm_Name' => null]])
            ->andWhere('LENGTH(trim(Dpm_Name)) > 0')
            ->groupBy(['Dpm_Name'])
            ->all();
        return $this->response(Error::STATUS_SUCCESS, $dpms);
    }

    public function actionList() {
        $query = User::find()->select([
                        'User_ID', 'User_Name', 'User_Desc', 'User_Sn', 'IcCard', 'IC_Status', 'User_Type',
                        'Dpm_Name','Create_Type',
                        "(case (isnull(Phone)) when true then 'unbind' else 'bind' end) as UnionId"
                    ]);
        // total
        $total = $query;
        $total->where('User_ID > 0');
        $count = $this->listQuery($total, $this->data);
        $users = $total->orderBy(['User_Name' => SORT_DESC])->all();
        // manager
        $manager = $query;
        $manager->where('User_ID > 0 and User_Name is not null');
        $managerCount = $this->listQuery($manager, $this->data);
        // ic
        $ic = $query;
        $ic->where('User_ID > 0 and IcCard is not null');
        $icCount = $this->listQuery($ic, $this->data);

        return $this->response(Error::STATUS_SUCCESS, [
            'count' => $count,
            'users' => $users,
            'manager' => $managerCount,
            'ic' => $icCount
        ]);
    }

    public function actionAuth()
    {
        if(!$this->isSuper)
            unset($this->data['mgrPrivilege'], $this->data['mgrDevices']);
        if(!isset($this->data['users']) ||
            (isset($this->data['devPrivilege']) && $this->data['devPrivilege'] == 0 && !isset($this->data['useDevices'])) ||
            (isset($this->data['mgrPrivilege']) && $this->data['mgrPrivilege'] == 0 && !isset($this->data['mgrDevices'])))
            return $this->response(Error::STATUS_PARAM_ERROR, $this->data);
        if(!preg_match('/^[0-9][0-9,]*$/', $this->data['users']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $db = Yii::$app->getDb();
        $users = $db->createCommand('select User_Desc,User_Name,User_ID from Users where User_ID in (' . $this->data['users'] . ')')
            ->queryAll();
        $ts = [];
        foreach ($users as $user) {
            if(!empty($user['User_Desc']))
                $ts[] = $user['User_Desc'];
            else if(!empty($user['User_Name']))
                $ts[] = $user['User_Name'];
            else
                $ts[] = $user['User_ID'];
        }

        if($ts)
            $userStr = implode("\n", $ts);
        else
            $userStr = '';
        if(!empty($userStr))
            $this->logDetail['users'] = mb_substr($userStr, 0, 980);

        if(isset($this->data['devPrivilege'])) {
            if ($this->data['devPrivilege'] == 1) {
                $db->createCommand('update Users set Dev_Privilege = 1 where User_ID in ( ' . $this->data['users'] . ');')
                    ->query();
            } else {
                $db->createCommand('update Users set Dev_Privilege = 0 where User_ID in ( ' . $this->data['users'] . ');')
                    ->query();
                if (!empty($this->data['useDevices']) && is_array($this->data ['useDevices'])) {
                    $devices = implode(',', $this->data['useDevices']);
                    if (!preg_match('/^[0-9][0-9,]+[0-9]$/', $devices))
                        return $this->response(Error::STATUS_PARAM_ERROR);
                    $db->createCommand("update UserDevices set Pri = Pri & 0xfe where UserID in (" . $this->data['users'] . ") and DevID not in ($devices);")
                        ->query();
                    $db->createCommand("insert into UserDevices (UserID, DevID, Pri)
                    select User_ID, DevID ,1  from DeviceList, Users where User_ID in (" . $this->data['users'] . ") and DevID in ($devices)
                    on duplicate key update Pri = Pri | 1;")
                        ->query();
                } else
                    $db->createCommand('update UserDevices set Pri = Pri & 0xfe where UserID in ( ' . $this->data['users'] . ');')
                        ->query();
            }
        }

        if(isset($this->data['mgrPrivilege'])) {
            if ($this->data['mgrPrivilege'] == 1) {
                $db->createCommand('update Users set Mgr_Privilege = 1 where User_ID in ( ' . $this->data['users'] . ');')
                    ->query();
            } else {
                $db->createCommand('update Users set Mgr_Privilege = 0 where User_ID in ( ' . $this->data['users'] . ');')
                    ->query();
                if (!empty($this->data['mgrDevices']) && is_array($this->data['mgrDevices'])) {
                    $devices = implode(',', $this->data['mgrDevices']);
                    if (!preg_match('/^[0-9][0-9,]+[0-9]$/', $devices))
                        return $this->response(Error::STATUS_PARAM_ERROR);
                    $db->createCommand("update UserDevices set Pri = Pri & 0xfd where UserID in (" . $this->data['users'] . ") and DevID not in ($devices);")
                        ->query();
                    $db->createCommand("insert into UserDevices (UserID, DevID, Pri)
                    select User_ID, DevID ,2  from DeviceList, Users where User_ID in (" . $this->data['users'] . ") and DevID in ($devices)
                    on duplicate key update Pri = Pri | 2;")
                        ->query();

                } else
                    $db->createCommand('update UserDevices set Pri = Pri & 0xfd where UserID in (' . $this->data['users'] . ');')
                        ->query();
            }
        }

        $db->createCommand('delete from UserDevices where Pri = 0;')->query();
        return $this->response(Error::STATUS_SUCCESS);
    }

    public function actionView($id)
    {
        if (Yii::$app->getUser()->getId() !== 0 && $id === 0)
            return $this->response(Error::STATUS_USER_NOT_EXISTS);
        $user = User::findOne(['User_ID' => $id]);
        if(is_null($user))
            return $this->response(Error::STATUS_USER_NOT_EXISTS);
        if (!is_null($user->UnionId))
            $user->UnionId = 'bind';
        else
            $user->UnionId = 'unbind';
        $useDevices = [];
        $mgrDevices = [];
        if($user->Dev_Privilege != 1) {
            $items = UserDevices::find()
                ->select(['DevID as device'])
                ->where(['UserID' => $id])
                ->andWhere('Pri & 1 = 1')
                ->asArray()
                ->all();
            foreach($items as $item)
                $useDevices[] = $item['device'];
        }
        if ($user->Mgr_Privilege != 1) {
            $items = UserDevices::find()
                ->select(['DevID as device'])
                ->where(['UserID' => $id])
                ->andWhere('Pri & 2 = 2')
                ->asArray()
                ->all();
            foreach ($items as $item)
                $mgrDevices[] = $item['device'];
        }
        return $this->response(Error::STATUS_SUCCESS, [
            'user' => $user,
            "useDevices" => $useDevices,
            "mgrDevices" => $mgrDevices,
        ]);
    }

    public function actionDelete()
    {
        if (!isset($this->data['User_ID']) || !preg_match('/^[0-9][0-9,]*$/', $this->data['User_ID']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        //'admin' can't to be deleted
        $db = Yii::$app->getDb();
        $users = $db->createCommand('select User_Desc, User_Name from Users where User_ID in (' . $this->data['User_ID'] . ')')
            ->queryAll();
        $names = [];
        foreach ($users as $user) {
            $names[] = is_null($user['User_Desc'])? $user['User_Name'] : $user['User_Desc'];
        }
        $title1 = implode(",", $names);
        if(mb_strlen($title1) > 256)
            $this->logDetail['users'] = mb_substr($title1, 0, 256) . "...";
        else
            $this->logDetail['users'] = $title1;
        if(mb_strlen($title1) > 64)
            $title1 = mb_substr($title1, 0, 64) . "...";
        $this->logTitle['title1'] = $title1;
        if ($this->isSuper)
            $db->createCommand('delete from Users where User_ID in (' . $this->data['User_ID'] . ') and User_ID > 0;')
                ->execute();
        else
            $db->createCommand('delete from Users where User_ID in (' . $this->data['User_ID'] . ') and User_ID > 0 and User_Name is null;')
                ->execute();

        $ret = $db->createCommand('select User_ID from Users where User_ID in (' . $this->data['User_ID'] . ')')
            ->queryAll();
        return $this->response(Error::STATUS_SUCCESS, ['fail' => $ret]);
    }

    private function UserNP($user)
    {
        unset($this->data['user']['UnionId']);
        unset($this->data['user']['User_ID']);
        unset($this->data['user']['Create_Type']);

        if($user->isNewRecord)
            $id = -1;
        else
            $id = $user->User_ID;
        if(!$this->isSuper) {
            unset ($this->data['user']['User_Name']);
            unset ($this->data['user']['New_Pass_Word']);
            unset($this->data['user']['privilege']);
            unset($this->data['user']['UsbKey']);
            unset($this->data['user']['Mgr_Privilege']);
        }
        else if(!empty($this->data['user']['User_Name']) ) {
            $count = User::find()->where(['not', ['User_ID' => $id]])
                ->andWhere(['User_Name' => $this->data['user']['User_Name']])
                ->count();
            if($count > 0)
                return $this->response(Error::STATUS_USER_USER_NAME_EXIST);
        } else if(isset($this->data['user']['User_Name']))
            $this->data['user']['User_Name'] = null;

        if(!empty($this->data['user']['IcCard']))
        {
            $count = User::find()->where(['not', ['User_ID' => $id]])
                ->andWhere(['IcCard' => $this->data['user']['IcCard']])
                ->count();
            if($count > 0)
            return $this->response(Error::STATUS_USER_ICCARD_EXIST);
        } else if(isset($this->data['user']['IcCard']))
            $this->data['user']['IcCard'] = null;

        if (!empty($this->data['user']['User_Sn'])) {
            $count = User::find()->where(['not', ['User_ID' => $id]])
                ->andWhere(['User_Sn' => $this->data['user']['User_Sn']])
                ->count();
            if ($count > 0)
                return $this->response(Error::STATUS_USER_USERSN_EXIST);
        } else if(isset($this->data['user']['User_Sn']))
            $this->data['user']['User_Sn']  = null;
        if (!(isset($this->data['user']['privilege']) && is_numeric($this->data['user']['privilege']))) {
            $this->data['user']['privilege'] = $user->privilege;
        }
        if (!(isset($this->data['user']['Priority']) && is_numeric($this->data['user']['Priority'])) || ($this->data['user']['Priority'] <= 0)) {
            $this->data['user']['Priority'] = 50;
        }
        if(!empty($this->data['user']['Phone'])){
            $count = Users::find()->where(['not', ['User_ID'=> $id]])
                     ->andWhere(['Phone'=>$this->data['user']['Phone']])->count();
            if($count>0)
                return $this->response(Error::STATUS_USER_PHONE_EXIST);
        }
        if($this->isSuper) {
            if (($this->data['user']['privilege'] & 0x100) == 0x100) {
                if (!empty($this->data['user']['UsbKey'])) {
                    $count = User::find()->where(['not', ['User_ID' => $id]])
                        ->andWhere(['UsbKey' => $this->data['user']['UsbKey']])
                        ->count();
                    if ($count > 0)
                        return $this->response(Error::STATUS_USER_USBKEY_EXIST);
                } else if (key_exists('UsbKey', $this->data['user']))
                    $this->data['user']['UsbKey'] = null;
            } else
                $this->data['user']['UsbKey'] = null;
        }
        $oldPri = $user->privilege;

        if ($user->load($this->data, 'user')) {
            if (!$this->isAdmin) {
                if ($oldPri & 0x40000000) {
                    $user->privilege |= 0x40000000;
                } else {
                    $user->privilege &= (~0x40000000);
                }
            }
            if (!empty($this->data['user']['New_Pass_Word']))
                $user->Pass_Word = User::mysql_password($this->data['user']['New_Pass_Word']);
            if ($user->save()) {
                $this->logTitle['title1'] = $user->User_Desc;
                $db = Yii::$app->getDb();
                if($user->isNewRecord)
                    $user = User::findOne(['User_ID' => $db->getLastInsertID()]);
                do {
                    Yii::$app->cache->delete(Common::USER_INFO_SALT . $user->User_ID);
                    if ($user->Dev_Privilege != 1 && isset($this->data['useDevices'])) {
                        if (!empty($this->data['useDevices']) && is_array($this->data['useDevices'])) {
                            $devices = implode(',', $this->data['useDevices']);
                            if (!preg_match('/^[0-9][0-9,]+[0-9]$/', $devices))
                                break;
                            $db->createCommand(
                                "update UserDevices set Pri = Pri & 0xfe where UserID=:userid and DevID not in ($devices);",
                                [':userid' => $user->User_ID]
                            )
                                ->query();
                            $db->createCommand('insert into UserDevices (UserID, DevID, Pri)
                                select :userid, DevID ,1 from DeviceList where DevID in (' . $devices . ')
                                on duplicate key update Pri = Pri | 1;', [':userid' => $user->User_ID])
                                ->query();
                        } else
                            $db->createCommand('update UserDevices set Pri = Pri & 0xfe where UserID=:userid;', [':userid' => $user->User_ID])
                                ->query();
                    }
                    if($this->isSuper && $user->Mgr_Privilege != 1 && isset($this->data['mgrDevices'])) {
                        if(!empty($this->data['mgrDevices']) && is_array($this->data['mgrDevices'])){
                            $devices = implode(',', $this->data['mgrDevices']);
                            if (!preg_match('/^[0-9][0-9,]+[0-9]$/', $devices))
                                break;
                            $db->createCommand(
                                'update UserDevices set Pri = Pri & 0xfd where UserID=:userid and DevID not in (' . $devices . ');',
                                [':userid' => $user->User_ID]
                            )
                                ->query();
                            $db->createCommand('insert into UserDevices (UserID, DevID, Pri)
                            select :userid, DevID ,2 from DeviceList where DevID in (' . $devices . ')
                            on duplicate key update Pri = Pri | 2;', [':userid' => $user->User_ID])
                                ->query();
                        }
                        else
                            $db->createCommand('update UserDevices set Pri = Pri & 0xfd where UserID=:userid;', [':userid' => $user->User_ID])
                                ->query();
                    }

                    $db->createCommand('delete from UserDevices where Pri = 0;')->query();
                } while (false);
                return $this->actionView($user->User_ID);
            }
        }
        return $this->response(Error::STATUS_FAILED, ['error' => $user->errors]);
    }

    public function actionUpdate($id)
    {
        if (!isset($this->data['user']))
            return $this->response(Error::STATUS_FAILED);
        if (Yii::$app->getUser()->getId() !== 0 && $id === 0)
            return $this->response(Error::STATUS_USER_NOT_EXISTS);
        $user = User::findOne(['User_ID' => $id]);
        if (is_null($user))
            return $this->response(Error::STATUS_USER_NOT_EXISTS);
        return $this->userNP($user);
    }

    public function actionNew()
    {
        if (!isset($this->data['user']))
            return $this->response(Error::STATUS_FAILED);
        $user = new User();
        return $this->userNP($user);
    }

    public function actionFind () {
        if (!Yii::$app->getUser()->identity->cloudLogin) {
            return $this->response(Error::STATUS_FAILED);
        }
        $data = Yii::$app->request->post('data');
        if (empty($data) || !is_array($data)) {
            return $this->response(Error::STATUS_PARAM_ERROR, $data);
        }
        $condition = null;
        foreach ($data as $key => $value) {
            $condition[$key] = $value;
        }
        $user = User::findOne($condition);
        if (empty($user)) {
            return $this->response(Error::STATUS_USER_NOT_EXISTS);
        }
        return $this->response(Error::STATUS_SUCCESS, $user);
    }

    public function actionCloudUnionidBind () {
        if (!Yii::$app->getUser()->identity->cloudLogin) {
            return $this->response(Error::STATUS_FAILED);
        }
        $data = Yii::$app->request->post('data');
        $unionId = hash('sha256', Common::UNIONID_MD5_PRESTR . $data['unionId']);
        $userId = $data['userId'];
        if (empty($unionId) || empty($userId))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $otherUser = User::findOne(['UnionId' => $unionId]);
        if (!empty($otherUser))
            return $this->response(Error::STATUS_USER_UNIONID_BOUND_OTHER,
                ['data' => ['userdesc' => $otherUser->User_Desc, 'username' => $otherUser->User_Name]]);
        $user = User::findOne(['User_ID' => $userId]);
        if (empty($user))
            return $this->response(Error::STATUS_USER_NOT_EXISTS);
        $user->UnionId = $unionId;
        $this->logTitle['username'] = !empty($user->User_Name) ? $user->User_Name : $user->User_Desc;
        if ($user->save()) {
            $this->logTitle['title1'] = !empty($user->User_Desc) ? $user->User_Desc : $user->User_Name;
            return $this->response(Error::STATUS_SUCCESS);
        }
        return $this->response(Error::STATUS_FAILED);
    }

    public function actionImportIcInfo () {
        $file = UploadedFile::getInstanceByName('csv-file');
        if ($file) {
            if ($file->type !== 'application/vnd.ms-excel' && $file->type !== 'application/octet-stream')
                return $this->response(Error::STATUS_FILE_TYPE_ERROR);
            $time = time();
            $file->saveAs('/tmp/' . $time . $file->name);
            $filePath = '/tmp/' . $time . $file->name;
            $handle = fopen($filePath, 'r');
            $this->logTitle['title1'] = $file->name;
            $errFlag = [];

            $dbt = Yii::$app->db->beginTransaction();
            $sql = 'INSERT INTO Users(User_Desc, IcCard, User_Sn, Phone, Dpm_Sn, Dpm_Name, User_Type) VALUES
                        (:User_Desc, :IcCard, :User_Sn, :Phone, :Dpm_Sn, :Dpm_Name, 2)
                        ON DUPLICATE KEY 
                        UPDATE User_Desc=VALUES(User_Desc), IcCard=VALUES(IcCard), User_Sn=VALUES(User_Sn), 
                        Phone=VALUES(Phone), Dpm_Sn=VALUES(Dpm_Sn), Dpm_Name=VALUES(Dpm_Name), User_Type=VALUES(User_Type);';
            $insert = $dbt->db->createCommand($sql);
            $insert->bindParam('User_Desc', $User_Desc);
            $insert->bindParam('IcCard', $IcCard);
            $insert->bindParam('User_Sn', $User_Sn);
            $insert->bindParam('Phone', $Phone);
            $insert->bindParam('Dpm_Sn', $Dpm_Sn);
            $insert->bindParam('Dpm_Name', $Dpm_Name);

            while (($fileOp = fgetcsv($handle, 99999, ',')) !== false) {
                if (count($fileOp) < 3) {
                    continue;
                }
                if (empty($fileOp[0]) || empty($fileOp[1]) || empty($fileOp[2])) {
                    continue;
                }
                $encoding = mb_detect_encoding($fileOp[0], 'UTF-8', true);
                if ($encoding !== 'UTF-8') {
                    return $this->response(Error::STATUS_FILE_ENCODING_ERROR);
                }
                $User_Desc = $fileOp[0];
                $User_Sn = $fileOp[1];
                $IcCard = $fileOp[2];
                $Phone = !empty($fileOp[3]) ? $fileOp[3] : null;
                $Dpm_Sn = !empty($fileOp[4]) ? $fileOp[4] : null;
                $Dpm_Name = !empty($fileOp[5]) ? $fileOp[5] : null;
                try {
                    $insert->execute();
                } catch (\Exception $e) {
                    $errFlag[] = $User_Desc;
                }
            }
            try {
                $dbt->commit();
            } catch (\Exception $e) {
                $errFlag[] = $User_Desc;
            }

            if (empty($errFlag)) {
                return $this->response(Error::STATUS_SUCCESS);
            }

            return $this->response(Error::STATUS_USER_IMPORT_FAILED);
        }
        return $this->response(Error::STATUS_FAILED);
    }
    public function actionGetUsersByPris () {
        $data = Yii::$app->getRequest()->post('data');
        if (empty($data) || !key_exists('pri', $data))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $pris = explode(',', $data['pri']);
        $priValue = 0;
        $getSuper = false;
        foreach ($pris as $pri) {
            if ($pri == 'system')
                $getSuper = true;
            else if (key_exists($pri, Common::USER_PRIVILEGE_SET))
                $priValue |= Common::USER_PRIVILEGE_SET[$pri];
        }
        $query = User::find()->select('User_ID, User_Name, User_Desc')
            ->where('User_ID > 0')
            ->andWhere("privilege & $priValue = $priValue")
            ->all();
        $supers = [];
        if ($getSuper) {
            $systemPri = Common::USER_PRIVILEGE_SET['system'];
            $supers = User::find()->where("privilege & $systemPri")->all();
        }
        $users = array_merge($query, $supers);

        return $this->response(Error::STATUS_SUCCESS, $users);
    }
}
