<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/1/24 14:54
 */
namespace Workerman\iDste;
use common\classes\Common;
use common\models\CallWaiter;
use common\models\DeviceList;
use common\models\UserDevices;
use common\models\Users;
use common\models\VDevice;
use common\models\VIpCall;
use common\models\VPlayingSessions;
use common\classes\Error;
use common\models\VDeviceMessage;
use Workerman\Lib\Timer;
use yii\db\Exception;
use Yii;

require_once __DIR__ . '/../Autoloader.php';
require_once __DIR__ . '/../../common/classes/System.php';
require_once __DIR__ . '/../../common/classes/HttpRequest.php';
require_once __DIR__ . '/../../common/classes/Error.php';

//websocket deamon
class WsDeamon
{
    const KEEPALIVE_TIMER_PERIOD = 1;
    const KEEPALIVE_PERIOD = 30;
    const KEEPALIVE_TIMER_CHANGE_THRESHOLD = 3;
    private $lastId = -1;
    private $lastAliveTime = 0;
    private $lastCheckTime = 0;
    private $clients = [];
    private $loginConns = [];
    private $status = [
        'connCnt' => [
            'device' => 0,
            'session' => 0,
            'ipcall' => 0,
            'player' => 0
        ]
    ];
    private $loginRequestController = ['device', 'session'];

    public function __construct($wsWorker)
    {
        $wsWorker->onWorkerStart = [$this, 'onWorkerStart'];
        $wsWorker->onConnect = [$this, 'onConnect'];
        $wsWorker->onMessage = [$this, 'onMessage'];
        $wsWorker->onClose = [$this, 'onClose'];
    }

    private function response($conn, $status, $data = null, $closeOnError = true) {
        $msg = Error::jsonResponse($status, $data);
        echo "send msg to client:$conn->id, status:$status\n";
        $conn->send($msg);
        if($closeOnError && $status != Error::STATUS_SUCCESS && $status < Error::STATUS_IDSTE_CODE_BEGIN)
            $conn->close();
        return '';
    }
    /*
     * websocket on worker start handle
     */
    public function onWorkerStart($worker)
    {
        echo "websocket worker start\n";
        $this->dbReconnect();

        Timer::add(1, [$this, 'deviceDaemon']);
        Timer::add(1, [$this, 'deviceStatisticDaemon']);
        Timer::add(1, [$this, 'sessionDaemon']);
        Timer::add(1, [$this, 'ipcallDaemon']);
        Timer::add(1, [$this, 'playerDaemon']);
        Timer::add(self::KEEPALIVE_TIMER_PERIOD, [$this, 'clientKeepAlive']);
        Timer::add(1, [$this, 'clientLoginCheck']);
        Timer::add(300, [$this, 'dbReconnect']);
        Timer::add(2, [$this, 'fetchPlayerStatus']);
    }

    /**
     * reconnect every 300 sec
     */
    public function dbReconnect()
    {
        while (true) {
            try {
                Yii::$app->db->close();
                Yii::$app->db->open();
                return;
            } catch (Exception $e) {
                echo "db error, wait. \n";
                sleep(1);
            }
        }
    }

    /*
     * websocket event handle: websocket client connect to server
     */
    public function onConnect($conn)
    {
        echo "onConnect:$conn->id\n";
        $this->loginConns[$conn->id] = [
            'at' => time(),
            'conn' => $conn
        ];
    }

    /**
     * @param $conn
     */
    public function clientLoginCheck() {
        $now = time();
        foreach ($this->loginConns as $id => $item) {
            if($now - $item['at'] > 5 || $now < $item['at']) {
                $this->response($item['conn'], Error::STATUS_ERROR_408);
            }
        }
    }

    public function siteController($conn, $action, $data) {
        switch ($action) {
            case 'login':
                if(empty($data) || empty($data->token) || !is_string($data->token) ||empty($data->type) || !is_string($data->type) || !in_array($data->type, ['login', 'unionid-login', 'oauth']))
                    return $this->response($conn, Error::STATUS_PARAM_ERROR);
                $cred = Yii::$app->getCache()->get(Common::LOGIN_TOKEN_CACHE_SALT . $data->token);
                if (empty($cred) || !isset($cred['user']))
                    return $this->response($conn, Error::STATUS_ERROR_401);
                $this->clients[$conn->id] = [
                    'conn' => $conn,
                    'uid' => $cred['user']->User_ID,
                    'user' => $cred['user'],
                    'token' => $data->token,
                    'type' => $data->type,
                    'init' => true,
                    'device' => [
                        'listen' => false,
                    ],
                    'session' => [
                        'listen' => false,
                        'init' => true,
                    ],
                    'ipcall' => [
                        'listen' => false,
                        'call' => null,
                        'waiter' => null
                    ],
                    'player' => [
                        'listen' => false,
                        'init' => true,
                    ]
                ];
                unset($this->loginConns[$conn->id]);
                echo "client:$conn->id send site/login request\n";
                return $this->response($conn, Error::STATUS_SUCCESS, ['time' => Yii::$app->base->date, 'weekday' => Yii::$app->base->weekday]);
                break;
        }
        return $this->response($conn, Error::STATUS_ERROR_404);
    }

    public function deviceController($conn, $action, $data) {
        switch ($action) {
            case 'listen':
                $this->clients[$conn->id]['device']['listen'] = true;
                $this->clients[$conn->id]['device']['init'] = true;
                $this->status['connCnt']['device']++;
                if(!empty($data) && isset($data->select) && is_string($data->select)) {
                    $this->clients[$conn->id]['device']['select'] =  preg_split('/\s*,\s*/', trim($data->select), -1, PREG_SPLIT_NO_EMPTY);
                }
                else
                    $this->clients[$conn->id]['device']['select'] = null;
                if(!empty($data) && property_exists($data, 'statistic') && $data->statistic !== false) {
                    $this->clients[$conn->id]['device']['statistic'] = ($data->statistic === true || $data->statistic === null || !is_numeric($data->statistic)) ? 'null' : $data->statistic;
                    $this->deviceStatisticDaemon($conn->id);
                }
                else
                    $this->clients[$conn->id]['device']['statistic'] = false;

                echo "client:$conn->id send device/listen request\n";
                return $this->response($conn, Error::STATUS_SUCCESS);
            case 'close':
                $this->clients[$conn->id]['device']['listen'] = false;
                echo "client:$conn->id send device/close request\n";
                return $this->response($conn, Error::STATUS_SUCCESS);
                break;
        }
        $this->response($conn,Error::STATUS_ERROR_404);
    }

    public function sessionController($conn, $action, $data) {
        switch ($action) {
            case 'listen':
                $this->clients[$conn->id]['session']['listen'] = true;
                $this->clients[$conn->id]['session']['init'] = true;
                $this->status['connCnt']['session']++;
                if(!empty($data) && isset($data->select) && is_string($data->select)) {
                    $columns =  preg_split('/\s*,\s*/', trim($data->select), -1, PREG_SPLIT_NO_EMPTY);
                    $this->clients[$conn->id]['session']['select'] = array_merge($columns, ['status', 'SessionID']);
                }
                else
                    $this->clients[$conn->id]['session']['select'] = null;

                echo "client:$conn->id send sission/listen request\n";
                return $this->response($conn, Error::STATUS_SUCCESS);
            case 'close':
                $this->clients[$conn->id]['session']['listen'] = false;
                echo "client:$conn->id send sission/close request\n";
                return $this->response($conn, Error::STATUS_SUCCESS);
                break;
        }
        $this->response($conn,Error::STATUS_ERROR_404);
    }


    public function ipcallController($conn, $action, $data) {
        switch ($action) {
            case 'listen':
                $this->clients[$conn->id]['ipcall']['listen'] = true;
                $this->clients[$conn->id]['ipcall']['waiter'] = null;
                $this->clients[$conn->id]['ipcall']['call'] = null;
                $this->status['connCnt']['ipcall']++;
                echo "client:$conn->id send sission/listen request\n";
                return $this->response($conn, Error::STATUS_SUCCESS);
            case 'close':
                $this->clients[$conn->id]['ipcall']['listen'] = false;
                $this->clients[$conn->id]['ipcall']['waiter'] = null;
                $this->clients[$conn->id]['ipcall']['call'] = null;
                echo "client:$conn->id send sission/close request\n";
                return $this->response($conn, Error::STATUS_SUCCESS);
                break;
        }
        $this->response($conn,Error::STATUS_ERROR_404);
    }


    public function playerController($conn, $action, $data) {
        switch ($action) {
            case 'listen':
                $this->clients[$conn->id]['player']['listen'] = true;
                $this->clients[$conn->id]['player']['init'] = true;
                $this->status['connCnt']['player']++;
                echo "client:$conn->id send player/listen request\n";
                return $this->response($conn, Error::STATUS_SUCCESS);
            case 'close':
                $this->clients[$conn->id]['player']['listen'] = false;
                echo "client:$conn->id send player/close request\n";
                return $this->response($conn, Error::STATUS_SUCCESS);
                break;
        }
        $this->response($conn,Error::STATUS_ERROR_404);
    }

    private function run($conn, $route, $data) {
        $route = preg_split('/\//', trim($route), 2);
        if(count($route) != 2)
            return $this->response($conn, Error::STATUS_PARAM_ERROR);

        if(in_array($route[0], $this->loginRequestController) && !isset($this->clients[$conn->id]))
            return $this->response($conn, Error::STATUS_ERROR_401);

        $controller = $route[0] . 'Controller';
        if(in_array($controller, get_class_methods($this)))
            call_user_func_array ([$this, $controller], [$conn, $route[1], $data]);
        else
            $this->response($conn,Error::STATUS_ERROR_404);
    }

    /*
     * send keepalive to all client
     */
    public function clientKeepAlive()
    {
        $timeNow = microtime(true);
        if ($timeNow - $this->lastCheckTime > self::KEEPALIVE_TIMER_CHANGE_THRESHOLD ||
            $this->lastCheckTime > $timeNow ||
            $timeNow - $this->lastAliveTime >= self::KEEPALIVE_PERIOD
        ) {
            $this->lastAliveTime = $timeNow;
            $msg = Error::jsonResponse(Error::STATUS_SUCCESS, ['time' => Yii::$app->base->date, 'weekday' => Yii::$app->base->weekday]);
            foreach ($this->clients as $conn) {
                $conn['conn']->send($msg);
            }
        }
        $this->lastCheckTime = $timeNow;
    }

    /*
     * websocket event handle:receive message from client
     * $data = [
     *      'route' => message route like 'site/login' or 'device/listen' or 'session/listen' and so on.
     *      'data' =>
     * ]
     */
    public function onMessage($conn, $data)
    {
        $msg = json_decode($data);
        if (!empty($msg) && isset($msg->route)) {
            if(empty($msg->data))
                $msg->data  = null;
            $this->run($conn, $msg->route, $msg->data);
        }
        else {
            $this->response($conn, Error::jsonResponse(Error::STATUS_ERROR_404));
        }
    }

    /*
     * websocket event handle: close
     */
    public function onClose($conn)
    {
        echo "close connect:$conn->id\n";

        if(!empty($this->clients[$conn->id]) && !is_null($this->clients[$conn->id]['ipcall']) && !is_null($this->clients[$conn->id]['ipcall']['waiter'])) {
            Yii::$app->db->createCommand("update IpCall set status='idle' where waiter=" . $this->clients[$conn->id]['ipcall']['waiter'] . ";")
                ->execute();
        }
        unset($this->clients[$conn->id]);
        unset($this->loginConns[$conn->id]);
        unset($this->deviceStatisticSended[$conn->id]);
    }

    private $userDevice = [];
    private function getUserDevice($uid)
    {
        $now = time();
        if(isset($this->userDevice[$uid]) && ($now - $this->userDevice[$uid]['at'] < 10) && ($now > $this->userDevice[$uid]['at']))
            return $this->userDevice[$uid]['devices'];

        $user = Users::find()->select(['User_ID', 'Mgr_Privilege', 'privilege'])->where(['User_ID' => $uid])->one();
        //user not exists or has no device privilege
        if (empty($user) || ($user->User_ID != 0 && (($user->privilege & 0x01) != 0x01)))
            return [];
        if ($user->User_ID == 0 || ($user->privilege & 0x40000000) || $user->Mgr_Privilege == 1) {
            $models = DeviceList::find()->select(['DevID'])->all();
        } else {
            $models = UserDevices::find()->select(['DevID'])
                ->where(['UserID' => $uid])
                ->andWhere('Pri & 2 = 2')
                ->all();
        }
        $devices = [];
        foreach ($models as $model) {
            $devices[] = $model->DevID;
        }
        $this->userDevice[$uid] = [
            'at' => $now,
            'devices' => $devices
        ];
        return $devices;
    }

    /*
     * ge message and send to clients
     */
    private $deviceCache = [];
    private $deviceStatistic = [];
    const DEVICE_STATISTIC_INIT = [
        'all' => 0,
        'off' => 0,
        'on' => 0,
        'disable' => 0,
        'offline' => 0
    ];

    /**
     * 定时获取分区下所有设备的统计数据
     */
    private function deviceClassesStatistic() {
        static $lastMessage = -1;
        $changed = [];
        if($this->lastId == $lastMessage)
            return $changed;
        //无消息且正常初始化后，数据未变更，退出
        $lastMessage = $this->lastId;
        foreach ($this->clients as $id => $client) {
            if (!$client['device']['listen'] || $client['device']['statistic'] === false)
                continue;
            $c = $client['device']['statistic'];
            $classes[$c] = $c;
        }
        if(!$classes)
            return $changed;
        foreach ($classes as $k => $c) {
            try {
                $sql = "select IF(ISNULL(power), 'offline', power) AS power, count(*) as cnt from v_device where sp_dev_belong_group(id, $c) group by power;";
                $items = Yii::$app->db->createCommand($sql)->queryAll();
            }catch (Exception $e) {continue;}
            $t = self::DEVICE_STATISTIC_INIT;
            foreach ($items as $item) {
                $t[$item['power']] = $item['cnt'];
            }
            $t['all'] = $t['off'] + $t['on'] + $t['offline'] + $t['disable'];
            if(!array_key_exists($k, $this->deviceStatistic) || array_diff_assoc($this->deviceStatistic[$k], $t))
                $changed[$k] = true;
            $this->deviceStatistic[$k] = $t;
        }
        return $changed;
    }

    /**
     * @param $classid  string
     * @param $userid   string
     * @param bool $force
     * @return array|null 变化时返回数组，未变化时返回空
     */
    private function deviceStatisticNormal($classid, $userid, $force = false) {
        //缓存最后检查的ID、结果和最后一次清理时间
        static $idcache = [];
        static $dcache = [];
        static $lastCleanAt = -1;
        //每隔5分钟清理一次缓存
        if(time() - $lastCleanAt > 300) {
            $idcache = [];
            $dcache = [];
            $lastCleanAt = time();
        }
        $key = $classid . '/' . $userid;
        if(array_key_exists($key, $idcache) && $idcache[$key] == $this->lastId && !$force)
            return null;
        try {
            $sql = "select IF(ISNULL(power), 'offline', power) AS power, count(*) as cnt from v_device " .
                "left join UserDevices on UserDevices.UserID=$userid and (UserDevices.Pri & 2) = 2 " .
                "where UserDevices.DevID=v_device.id and sp_dev_belong_group(id, $classid) group by power;";
            $items = Yii::$app->db->createCommand($sql)->queryAll();
        } catch (Exception $e) {return null;}
        $idcache[$key] = $this->lastId;
        $t = self::DEVICE_STATISTIC_INIT;
        foreach ($items as $item)
            $t[$item['power']] = $item['cnt'];
        $t['all'] = $t['off'] + $t['on'] + $t['offline'] + $t['disable'];
        if(!$force && array_key_exists($key, $dcache) && !array_diff_assoc($dcache[$key], $t))
            return null;
        $dcache[$key] = $t;
        return $t;
    }

    /**
     * @param int $cid 非-1时仅检查指定client id的数据
     */
    public function deviceStatisticDaemon($cid = -1) {
        static $sended = [];
        static $changed = [];
        //check_id不为-1时使用存储的数据，否则执行一次分区统计
        if($cid == -1)
            $changed = $this->deviceClassesStatistic();
        else
            unset($sended[$cid]);
        foreach ($this->clients as $id => $client) {
            if (!$client['device']['listen'] || $client['device']['statistic'] === false)
                continue;
            //只检查指定客户端数据时
            if($cid != -1 && $cid != $id)
                continue;
            $c = $client['device']['statistic'];
            //有变更或未发送过统计数据
            /* @var $user Users */
            $user = $client['user'];
            if($user->User_ID == 0 || $user->Mgr_Privilege == 1 || ($user->privilege & 0x40000000) == 0x40000000) {
                //超级管理员或有所有设备管理权限者使用分区统计数据
                if (array_key_exists($c, $this->deviceStatistic) && (array_key_exists($c, $changed) || !array_key_exists($id, $sended))) {
                    $this->response($client['conn'], Error::STATUS_DEVICE_STATISTIC_CHANGED, $this->deviceStatistic[$c]);
                    $sended[$id] = true;
                }
            }
            else {
                //普通管理员需要根据UserDevices权限统计
                $ret = $this->deviceStatisticNormal($c, $user->User_ID, ($cid != -1 && $cid == $id));
                if($ret)
                    $this->response($client['conn'], Error::STATUS_DEVICE_STATISTIC_CHANGED, $ret);
            }
        }
    }
    
    private function getProj2OpenFlag() {
        static $proj2CheckAt = -1;
        static $proj2Open = null;
        if(time() - $proj2CheckAt > 30) {
            Yii::$app->base->baseGet(Common::BASEINFO_SECOND_PROJECTOR_SUPPORT, $proj2Name, $proj2Open);
            $proj2CheckAt = time();
        }
        return $proj2Open;
    }

    public function deviceDaemon()
    {
        if($this->status['connCnt']['device'] == 0)
            return;
        $models = VDeviceMessage::find()
            ->where('message > ' . $this->lastId)
            ->orderBy(['message' => SORT_DESC])
            ->groupBy(['id'])
            ->asArray()
            ->all();

        if(empty($models))
            return;

        if ($this->lastId < $models[0]['message'])
            $this->lastId = $models[0]['message'];
        $proj2Open = $this->getProj2OpenFlag();
        $msgs = [];
        foreach ($models as $model) {
            $model['actualSession'] = is_null($model['actualSession']) ? null : intval($model['actualSession']);
            $model['audioVol'] = is_null($model['audioVol']) ? null : intval($model['audioVol']);
            $model['fault'] = is_null($model['fault']) ? null : intval($model['fault']);
            $model['micVol'] = is_null($model['micVol']) ? null : intval($model['micVol']);
            $model['parent'] = is_null($model['parent']) ? null : intval($model['parent']);
            $model['shouldSession'] = is_null($model['shouldSession']) ? null : intval($model['shouldSession']);
            $model['projector2'] = ($proj2Open !== 1 && !is_null($model['projector2'])) ? null : $model['projector2'];
            unset($model['message']);
            $msgs[$model['id']] = $model;
        }
        $connCnt = 0;
        foreach ($this->clients as $clientid => $client) {
            if (!$client['device']['listen'])
                continue;
            $connCnt++;
            $uid = $client['uid'];
            $devices = $this->getUserDevice($uid);
            if (empty($devices))
                continue;
            $ret = [];
            $retKey = 'diff';
            foreach ($devices as $id) {
                if(!isset($msgs[$id])) {
                    if(isset($this->deviceCache[$id]) && $client['device']['init']) {
                        $retKey = 'flush';
                        $diff = $this->deviceCache[$id];
                    }
                    else
                        continue;
                }
                else {
                    if (!isset($this->deviceCache[$id]) || $client['device']['init']) {
                        $diff =$msgs[$id];
                    } else {
                        $diff = array_diff_assoc($msgs[$id], $this->deviceCache[$id]);
                        if (empty($diff))
                            continue;
                    }
                }
                if(empty($client['device']['select'])) {
                    $diff['status'] = 'change';
                    $diff['id'] = $id;
                    $ret[] = $diff;
                }
                else {
                    $diffret = [];
                    foreach ($client['device']['select'] as $colname) {
                        if(array_key_exists($colname, $diff))
                            $diffret[$colname] = $diff[$colname];
                    }
                    if(!empty($diffret)) {
                        $diffret['status'] = 'change';
                        $diffret['id'] = $id;
                        $ret[] = $diffret;
                    }
                }
            }
            $this->clients[$clientid]['device']['init'] = false;
            if (empty($ret))
                continue;
            $this->response($client['conn'], Error::STATUS_DEVICE_CHANGED, [$retKey => $ret]);
        }
        foreach ($msgs as $id => $msg)
            $this->deviceCache[$id] = $msg;
        $this->status['connCnt']['device'] = $connCnt;
        if($connCnt == 0)
            $this->deviceCache = [];
    }

    /**
     * get session status and send to client
     */
    private $sessionCache = [];
    public function sessionDaemon(){
        if($this->status['connCnt']['session'] == 0)
            return;
        $models = VPlayingSessions::find()
            ->select(['SessionID', 'FileName', 'CurrTotalProgress', 'Pause', 'PlayTime', 'WeekDay', 'Template', 'BrdMode', 'SessionType', 'StreamType'])
            ->indexBy('SessionID')
            ->orderBy(['PlayTime' => SORT_ASC])
            ->asArray()
            ->all();
        $diffs = [];
        foreach ($models as $id => $model) {
            $model['CurrTotalProgress'] = is_null($model['CurrTotalProgress']) ? 0 : intval($model['CurrTotalProgress']);
            $model['Pause'] = is_null($model['Pause']) ? null : intval($model['Pause']);
            $model['SessionID'] = intval($model['SessionID']);
            if(isset($this->sessionCache[$id])){
                if($this->sessionCache[$id]['Pause'] == $model['Pause'])
                     $this->sessionCache[$id]['CurrTotalProgress'] = $model['CurrTotalProgress'];
                $diff = array_diff_assoc($model, $this->sessionCache[$id]);
                if (empty($diff))
                    continue;
                $diff['status'] = 'change';
                $diff['PlayTime'] = $model['PlayTime'];
                $diff['CurrTotalProgress'] = $model['CurrTotalProgress'];
                $diff['SessionID'] = $id;
            }else {
                $diff = $model;
                $diff['status'] = 'change';
            }
            $diffs[] = $diff;
            $this->sessionCache[$id] = $model;
        }

        foreach ($this->sessionCache as $id => $model) {
            if(!isset($models[$id])) {
                $diffs[] = ['status' => 'stop', 'SessionID' => intval($id), 'PlayTime' => $model['PlayTime']];
                unset($this->sessionCache[$id]);
            }
        }

        $connCnt = 0;
        foreach ($this->clients as $id => $client) {
            if(!$client['session']['listen'])
                continue;
            $connCnt++;
            if(count($this->sessionCache) > 0 && ($client['session']['init'] || (time() % 63 == 0))){
                $this->clients[$id]['session']['init'] = false;
                if(empty($client['session']['select'])) {
                    $this->response($client['conn'], Error::STATUS_SESSION_STATUS_CHANGED, ['flush' => array_values($this->sessionCache)]);
                    continue;
                }
                else {
                    $retDiffs = array_values($this->sessionCache);
                    $retKey = 'flush';
                }
            }
            else if (!empty($diffs)) {
                if(empty($client['session']['select'])) {
                    $this->response($client['conn'], Error::STATUS_SESSION_STATUS_CHANGED, ['diff' => $diffs]);
                    continue;
                }
                else {
                    $retDiffs = $diffs;
                    $retKey = 'diff';
                }
            }
            else
                continue;
            $ret = [];
            foreach ($retDiffs as $diff) {
                $msg = [];
                foreach ($client['session']['select'] as $colname) {
                    if(array_key_exists($colname, $diff))
                        $msg[$colname] = $diff[$colname];
                }
                if(!empty($msg))
                    $ret[] = $msg;
            }
            if(!empty($ret))
                $this->response($client['conn'], Error::STATUS_SESSION_STATUS_CHANGED, [$retKey => $ret]);
        }
        $this->status['connCnt']['session'] = $connCnt;
        if($connCnt == 0)
            $this->sessionCache = [];
    }

    private $playerStatusCache = [];
    public function fetchPlayerStatus () {
        if($this->status['connCnt']['player'] == 0) {
            $this->playerStatusCache = [];
            return;
        }
        $models = Yii::$app->db
            ->createCommand("select PlayerID, PlayType, IpAddress from PlayerStatus where PlayType = 1;")
            ->queryAll();
        $models1 = Yii::$app->db
            ->createCommand("select id as PlayerID, NULL as PlayType, ipaddress as IpAddress from v_device where id like '49%';")
            ->queryAll();
        $models = array_merge($models, $models1);
        foreach ($models as $model) {
            $ip = $model['IpAddress'];
            if (empty($ip)) {
                $this->playerStatusCache[$model['PlayerID']] = ['fileName' => '', 'fileId' => null, 'dirId' => null];
                continue;
            }
            $url = "http://webApp:webApp@" . $ip . "/cgi-bin/getplayerst.cgi";
            $ch = curl_init ();
            curl_setopt ( $ch, CURLOPT_URL, $url );
            curl_setopt ( $ch, CURLOPT_RETURNTRANSFER, 1 );
            curl_setopt ( $ch, CURLOPT_CONNECTTIMEOUT, 15 );
            curl_setopt ( $ch, CURLOPT_HEADER, false );
            $playerSt = curl_exec ( $ch );
            $playerSt = str_replace(["\n", ",}", ',]'], ['', '}', ']'], $playerSt);
            curl_close ( $ch );
            $playerStatus = json_decode($playerSt, true);
            if (!empty($playerStatus) && is_array($playerStatus)) {
                $this->playerStatusCache[$model['PlayerID']] = $playerStatus[0];
            }
        }

    }


    /**
     * fetch player status and send to client
     */
    private $playerCache = [];
    public function playerDaemon(){
        if($this->status['connCnt']['player'] == 0)
            return;
        $lists1 = Yii::$app->db->createCommand('select a.PlayerID, a.SessionID as RealSessionID, a.SessionName, a.Priority, a.AudioGain, a.Owner, a.OwnerName, a.StreamType, a.SessionType,
                                                    c.SessionID, c.ScheduleID, a.PlayerAddress as IpAddress, a.PlayType, a.PlayerType from v_session_info a
                                                    left join PlayerStatus c on c.SessionID  = a.SessionID
                                                    where a.PlayerID is not null;')
            ->queryAll();
        $lists2 = Yii::$app->db->createCommand("select a.id as PlayerID, a.actualSession as RealSessionID, a.SessionName, a.ModePriority as Priority, a.audioVol as AudioGain, NULL as Owner, NULL as OwnerName,
                                                    1 as StreamType, NULL as SessionType,
                                                    NULL as SessionID, NULL as ScheduleID, a.ipaddress as IpAddress, NULL as PlayType, NULL as PlayerType from v_device a
                                                    where a.id like '49%';")
            ->queryAll();
        $models = array_merge($lists1, $lists2);

        $diffs = [];
        $list = [];
        foreach ($models as $model) {
            $id = $model['PlayerID'];
            $list[$id] = true;
            $model['SessionID'] = intval($model['SessionID']);
            $model['RealSessionID'] = intval($model['RealSessionID']);
            $model['ScheduleID'] = intval($model['ScheduleID']);
            $model['PlayerType'] = intval($model['PlayerType']);

            if (!empty($this->playerStatusCache[$id])) {
                $model = array_merge($model, $this->playerStatusCache[$id]);
            }

            if(isset($this->playerCache[$id])){
                $diff = array_diff_assoc($model, $this->playerCache[$id]);
                if (empty($diff))
                    continue;
                $diff['status'] = 'change';
                $diff['PlayerID'] = $id;
                $diffs[] = $diff;
            }else {
                $model['status'] = 'change';
                $diffs[] = $model;
            }
            $this->playerCache[$id] = $model;
        }

        foreach ($this->playerCache as $id => $model) {
            if(!isset($list[$id])) {
                $diffs[] = ['status' => 'delete', 'PlayerID' => '' . $id];
                unset($this->playerCache[$id]);
            }
        }

        $connCnt = 0;
        foreach ($this->clients as $id => $client) {
            if(!$client['player']['listen'])
                continue;
            $connCnt++;
            if(count($this->playerCache) > 0 && ($client['player']['init'] || (time() % 61 == 0))){
                $this->response($client['conn'], Error::STATUS_PLAYER_STATUS_CHANGED, ['flush' => array_values($this->playerCache)]);
                $this->clients[$id]['player']['init'] = false;
            }
            else if (!empty($diffs))
                $this->response($client['conn'], Error::STATUS_PLAYER_STATUS_CHANGED, ['diff' => array_values($diffs)]);
        }
        $this->status['connCnt']['player'] = $connCnt;
        if($connCnt == 0)
            $this->playerCache = [];
    }
    /**
     * get ipcall recoder
     */
    public function ipcallDaemon(){
        if($this->status['connCnt']['ipcall'] == 0)
            return;

        $connCnt = 0;
        foreach ($this->clients as $id => $client){
            if(!$client['ipcall']['listen'])
                continue;
            $connCnt++;
            Yii::$app->db
                ->createCommand("insert into CallWaiter (user, session) values(" . $client['uid'] . ", '" . $client['token'] . "') on duplicate key update updateAt= now()")
                ->execute();
            if(is_null($this->clients[$id]['ipcall']['waiter'])) {
                $waiter = CallWaiter::findOne(['user' => $client['uid'], 'session' => $client['token']]);
                if(empty($waiter))
                    continue;
                else
                    $this->clients[$id]['ipcall']['waiter'] = $waiter->id;
            }
            $call = VIpCall::find()
                ->select(['id', 'device', 'deviceName', 'waiter', 'waiterName', 'ringing', 'status', 'direction', 'wsPort'])
                ->where("(`waiter` is null and `status`='ringing') or (`waiter`=" . $this->clients[$id]['ipcall']['waiter'] . " and `status` <> 'idle')")
                ->orderBy(['waiter' => SORT_DESC])
                ->asArray()
                ->one();

            $oldCall = $this->clients[$id]['ipcall']['call'];
            if(!empty($call)) {
                $call['id'] = intval($call['id']);
                $call['wsPort'] = intval($call['wsPort']);
                if(empty($oldCall) || $oldCall['id'] != $call['id'] || (!empty(array_diff_assoc($call, $oldCall)))) {
                    $this->response($client['conn'], Error::STATUS_IPCALL_STATE_CHANGED, [$call]);
                }
            }
            else if(!empty($oldCall)){
                $this->response($client['conn'], Error::STATUS_IPCALL_STATE_CHANGED, null);
            }
            $this->clients[$id]['ipcall']['call'] = $call;
        }
        $this->status['connCnt']['ipcall'] = $connCnt;
    }
}
